@@ -1,4 +1,4 @@
-/* KDevelop xUnit plugin
+/* KDevelop coverage plugin
  *
  * Copyright 2008 Manuel Breugelmans <mbr.nxi@gmail.com>
  *
@@ -30,11 +30,9 @@
 #include <KAboutData>
 #include <KActionCollection>
 #include <KActionMenu>
-#include <KDebug>
 #include <KLocale>
 #include <KPluginFactory>
 #include <KPluginLoader>
-#include <QDir>
 
 using KDevelop::ICore;
 using KDevelop::IUiController;
@@ -62,7 +60,6 @@ CoveragePlugin::~CoveragePlugin()
 
 QFileInfoList CoveragePlugin::findGcdaFilesIn(QDir& dir)
 {
-    kDebug() << dir.absolutePath();
     QFileInfoList gcdaFiles;
     QDir current(dir);
     current.setFilter(QDir::Dirs | QDir::NoDotAndDotDot | QDir::Readable | QDir::Writable);
