@@ -46,13 +46,26 @@
 #include <interfaces/idocumentcontroller.h>
 #include "implementationhelperitem.h"
 #include <qtfunctiondeclaration.h>
+#include <language/duchain/duchainutils.h>
+
+// #define ifDebug(x) x
 
 #define LOCKDUCHAIN     DUChainReadLocker lock(DUChain::lock())
 
+QString last5Lines(QString str) {
+  QStringList lines = str.split("\n");
+  if(lines.count() < 5)
+    return str;
+  else
+    return QStringList(lines.mid(lines.count()-5, 5)).join("\n");
+}
+
 ///If this is enabled, KDevelop corrects wrong member access operators like "." on a pointer automatically
 const bool assistAccessType = true;
 ///If this is enabled, no chain of useless argument-hints for binary operators is created.
 const bool noMultipleBinaryOperators = true;
+///Whether only items that are allowed to be accessed should be shown
+const bool doAccessFiltering = true;
 #ifdef TEST_COMPLETION
 //Stub implementation that does nothing
 QList<KDevelop::CompletionTreeItemPointer> missingIncludeCompletionItems(QString expression, QString displayTextPrefix, Cpp::ExpressionEvaluationResult expressionResult, KDevelop::DUContext* context, int argumentHintDepth, bool namespaceAllowed) {
@@ -124,14 +137,20 @@ bool isKeyword(QString str) {
 
 int completionRecursionDepth = 0;
 
-CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QString& text, const QString& followingText, int depth, const QStringList& knownArgumentExpressions, int line ) : KDevelop::CodeCompletionContext(context, text, depth), m_memberAccessOperation(NoMemberAccess), m_knownArgumentExpressions(knownArgumentExpressions), m_contextType(Normal), m_onlyShowTypes(false)
+CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QString& text, const QString& followingText, int depth, const QStringList& knownArgumentExpressions, int line ) : KDevelop::CodeCompletionContext(context, text, depth), m_memberAccessOperation(NoMemberAccess), m_knownArgumentExpressions(knownArgumentExpressions), m_contextType(Normal), m_onlyShowTypes(false), m_onlyShowSignals(false), m_onlyShowSlots(false), m_pointerConversionsBeforeMatching(0)
 {
   if(m_duContext) {
     LOCKDUCHAIN;
     if((m_duContext->type() == DUContext::Class || m_duContext->type() == DUContext::Namespace || m_duContext->type() == DUContext::Global))
       m_onlyShowTypes = true;
+
+    Declaration* classDecl = Cpp::localClassFromCodeContext(m_duContext.data());
+    if(classDecl) {
+      kDebug() << "local class:" << classDecl->qualifiedIdentifier().toString(  );
+      m_localClass = DUContextPointer(classDecl->internalContext());
+    }
   }
-  
+
   m_followingText = followingText;
 
   if(depth > 10) {
@@ -155,14 +174,17 @@ CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QSt
     return;
   }
 
-  ifDebug( log( "non-processed text: " + m_text ); )
-   preprocessText( line );
-
-   m_text = clearComments( m_text );
-   m_text = clearStrings( m_text );
+//   ifDebug( log( "non-processed text: " + m_text ); )
+  if(depth == 0) {
+    preprocessText( line );
+    m_text = clearComments( m_text );
+    m_text = clearStrings( m_text );
+  }
+    
    m_text = stripFinalWhitespace( m_text );
+   m_text = last5Lines(m_text);
 
-  ifDebug( log( "processed text: " + m_text ); )
+  ifDebug( log( QString("depth %1").arg(depth) + " end of processed text: " + m_text ); )
 
   ///@todo template-parameters
 
@@ -188,35 +210,67 @@ CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QSt
     m_text = m_text.left( m_text.length()-2 );
   }
 
+  if( m_text.endsWith('(') || m_text.endsWith( '<' ) ) {
+    if( depth == 0 ) {
+      //The first context should never be a function-call context, so make this a NoMemberAccess context and the parent a function-call context.
+      m_parentContext = new CodeCompletionContext( m_duContext, m_text, QString(), depth+1 );
+      m_text.clear();
+    }else{
+      ExpressionParser expressionParser;
+      
+      if(m_text.endsWith('(')) {
+        m_contextType = FunctionCall;
+        m_memberAccessOperation = FunctionCallAccess;
+        m_text = m_text.left( m_text.length()-1 );
+      }else{
+        //We need to check here whether this really is a template access, or whether it is a "smaller" operator,
+        //which is handled below
+        int start_expr = Utils::expressionAt( m_text, m_text.length()-1 );
+
+        QString expr = m_text.mid(start_expr, m_text.length() - start_expr - 1).trimmed();        
+        
+        Cpp::ExpressionEvaluationResult result = expressionParser.evaluateExpression(expr.toUtf8(), m_duContext);
+        if(result.isValid() && (!result.isInstance || result.type.type().cast<FunctionType>())) {
+          m_memberAccessOperation = TemplateAccess;
+          m_text = m_text.left( m_text.length()-1 );
+        }
+      }
+
+      ///Compute the types of the argument-expressions
+
+      for( QStringList::const_iterator it = m_knownArgumentExpressions.begin(); it != m_knownArgumentExpressions.end(); ++it )
+        m_knownArgumentTypes << expressionParser.evaluateExpression( (*it).toUtf8(), m_duContext );
+    }
+  }
+
   if( endsWithOperator( m_text ) ) {
     if( depth == 0 ) {
       //The first context should never be a function-call context, so make this a NoMemberAccess context and the parent a function-call context.
       m_parentContext = new CodeCompletionContext( m_duContext, m_text, QString(), depth+1 );
-      return;
+      m_text.clear();
+    }else{
+      m_memberAccessOperation = FunctionCallAccess;
+      m_contextType = BinaryOperatorFunctionCall;
+      m_operator = getEndFunctionOperator(m_text);
+      m_text = m_text.left( m_text.length() - getEndOperator(m_text).length() );
     }
-    m_memberAccessOperation = FunctionCallAccess;
-    m_contextType = BinaryOperatorFunctionCall;
-    m_operator = getEndFunctionOperator(m_text);
-    m_text = m_text.left( m_text.length() - getEndOperator(m_text).length() );
   }
 
-  if( m_text.endsWith('(') ) {
-    if( depth == 0 ) {
-      //The first context should never be a function-call context, so make this a NoMemberAccess context and the parent a function-call context.
-      m_parentContext = new CodeCompletionContext( m_duContext, m_text, QString(), depth+1 );
-      return;
+  ///Eventually take preceding "*" and/or "&" operators and use them for pointer depth conversion of the completion items
+  while(parentContext() && parentContext()->m_memberAccessOperation == FunctionCallAccess && parentContext()->m_contextType == BinaryOperatorFunctionCall && parentContext()->m_expression.isEmpty()) {
+    if(parentContext()->m_operator == "*") {
+      --m_pointerConversionsBeforeMatching;
+      setParentContext(parentContext()->m_parentContext);
+      continue;
     }
-    m_contextType = FunctionCall;
-    m_memberAccessOperation = FunctionCallAccess;
-    m_text = m_text.left( m_text.length()-1 );
-
-    ///Compute the types of the argument-expressions
-    ExpressionParser expressionParser;
-
-    for( QStringList::const_iterator it = m_knownArgumentExpressions.begin(); it != m_knownArgumentExpressions.end(); ++it )
-      m_knownArgumentTypes << expressionParser.evaluateExpression( (*it).toUtf8(), m_duContext );
+    if(parentContext()->m_operator == "&") {
+      ++m_pointerConversionsBeforeMatching;
+      setParentContext(parentContext()->m_parentContext);
+      continue;
+    }
+    break;
   }
-
+  
   ///Now find out where the expression starts
 
   /**
@@ -266,7 +320,7 @@ CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QSt
   }
 
   ///Handle recursive contexts(Example: "ret = function1(param1, function2(" )
-  if( expressionPrefix.endsWith('(') || expressionPrefix.endsWith(',') ) {
+  if( expressionPrefix.endsWith( '<' ) || expressionPrefix.endsWith('(') || expressionPrefix.endsWith(',') ) {
     log( QString("Recursive function-call: Searching parent-context in \"%1\"").arg(expressionPrefix) );
     //Our expression is within a function-call. We need to find out the possible argument-types we need to match, and show an argument-hint.
 
@@ -283,14 +337,19 @@ CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QSt
   }
 
   ///Handle signal/slot access
-  if(m_parentContext && parentContext()->memberAccessOperation() == FunctionCallAccess) {
+  if(depth == 0 && m_parentContext && parentContext()->memberAccessOperation() == FunctionCallAccess) {
     LOCKDUCHAIN;
-    
+
     if(parentContext()->functionName() == "SIGNAL" || parentContext()->functionName() == "SLOT") {
-      kDebug() << "skipping parents";
-      m_parentContext = parentContext()->m_parentContext; //Ignore the SIGNAL and SLOT parts
+      if(parentContext()->functionName() == "SIGNAL")
+        m_onlyShowSignals = true;
+      if(parentContext()->functionName() == "SLOT") {
+        m_onlyShowSlots = true;
+      }
+      
+      setParentContext(KSharedPtr<KDevelop::CodeCompletionContext>(parentContext()->parentContext()));
     }
-    
+
     if(m_parentContext && parentContext()->memberAccessOperation() == FunctionCallAccess) {
       foreach(const Cpp::OverloadResolutionFunction &function, parentContext()->functions()) {
         if(function.function.declaration() && (function.function.declaration()->qualifiedIdentifier().toString() == "QObject::connect" || function.function.declaration()->qualifiedIdentifier().toString() == "QObject::disconnect")) {
@@ -301,10 +360,10 @@ CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QSt
               m_memberAccessOperation = SignalAccess;
             }else if(funType->arguments()[function.matchedArguments] && funType->arguments()[function.matchedArguments]->toString() == "const char*") {
               m_memberAccessOperation = SlotAccess;
-              
+
               if(parentContext()->m_knownArgumentExpressions.size() > 1) {
                 QString connectedSignal = parentContext()->m_knownArgumentExpressions[1];
-                if(connectedSignal.startsWith("SIGNAL(") && connectedSignal.endsWith(")")) {
+                if(connectedSignal.startsWith("SIGNAL") && connectedSignal.endsWith(")") && connectedSignal.length() > 8) {
                   connectedSignal = connectedSignal.mid(7);
                   connectedSignal = connectedSignal.left(connectedSignal.length()-1);
                   //Now connectedSignal is something like myFunction(...), and we want the "...".
@@ -314,7 +373,7 @@ CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QSt
                 }
               }
             }
-            
+
             if(m_memberAccessOperation == SignalAccess || m_memberAccessOperation == SlotAccess) {
               if(function.matchedArguments == 2) {
                 //The function that does not take the target-argument is being used
@@ -331,6 +390,9 @@ CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QSt
       }
     }
   }
+  
+  if(depth == 0 && parentContext() && parentContext()->memberAccessOperation() == TemplateAccess)
+    m_onlyShowTypes = true; ///@todo Check the current item whether it's int or bool, and then also allow constants/enumerators
 
   ///Handle overridden binary operator-functions
   if( endsWithOperator(expressionPrefix) || expressionPrefix.endsWith("return") ) {
@@ -340,13 +402,13 @@ CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QSt
 
   ///Now care about m_expression. It may still contain keywords like "new "
 
-  bool isEmit = false, isThrow = false;
+  bool isThrow = false;
 
   QString expr = m_expression.trimmed();
 
   if( expr.startsWith("emit") )  {
-    isEmit = true; //When isEmit is true, we should filter the result so only signals are left
     expr = expr.right( expr.length() - 4 );
+    m_onlyShowSignals = true;
   }
   if( expr.startsWith("return") )  {
     expr = expr.right( expr.length() - 6 );
@@ -357,11 +419,24 @@ CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QSt
       m_memberAccessOperation = ReturnAccess;
     }
   }
+  if( expr.startsWith("delete") )  {
+    expr = expr.right( expr.length() - 6 );
+    if(!expr.isEmpty() || depth == 0) {
+      //Create a new context for the "delete"
+      m_parentContext = new CodeCompletionContext( m_duContext, "delete", QString(), depth+1 );
+    }else{
+      //m_memberAccessOperation = DeleteAccess;
+    }
+  }
   if( expr.startsWith("throw") )  {
     isThrow = true;
     expr = expr.right( expr.length() - 5 );
   }
-
+  if( expr.startsWith("new") ) {
+    m_onlyShowTypes = true;
+    m_pointerConversionsBeforeMatching = 1;
+    expr = expr.right( expr.length() - 3 );
+  }
   ExpressionParser expressionParser/*(false, true)*/;
 
   ifDebug( kDebug(9007) << "expression: " << expr; )
@@ -373,6 +448,8 @@ CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QSt
       if( m_memberAccessOperation != StaticMemberChoose ) {
         log( QString("expression \"%1\" could not be evaluated").arg(expr) );
         m_valid = false;
+        if(m_memberAccessOperation == FunctionCallAccess)
+          m_functionName = m_expression;
         return;
       } else {
         //It may be an access to a namespace, like "MyNamespace::".
@@ -471,6 +548,9 @@ CodeCompletionContext::CodeCompletionContext(DUContextPointer context, const QSt
     case FunctionCallAccess:
       processFunctionCallAccess();
     break;
+    case TemplateAccess:
+      //Nothing to do for now
+    break;
   }
 }
 
@@ -606,10 +686,38 @@ QList<DUContext*> CodeCompletionContext::memberAccessContainers() const {
       }
     }
   }
+  
+//   foreach(DUContext* context, ret) {
+//     kDebug() << "member-access container:" << context->url().str() << context->range().textRange() << context->scopeIdentifier(true).toString();
+//   }
 
   return ret;
 }
 
+KDevelop::IndexedType CodeCompletionContext::applyPointerConversionForMatching(KDevelop::IndexedType type) const {
+  if(m_pointerConversionsBeforeMatching == 0)
+    return type;
+  AbstractType::Ptr t = type.type();
+  if(!t)
+    return KDevelop::IndexedType();
+  
+  if(m_pointerConversionsBeforeMatching > 0) {
+    for(int a = 0; a < m_pointerConversionsBeforeMatching; ++a) {
+      t = TypeUtils::increasePointerDepth(t);
+      if(!t)
+        return IndexedType();
+    }
+  }else{
+    for(int a = m_pointerConversionsBeforeMatching; a < 0; ++a) {
+      t = TypeUtils::decreasePointerDepth(t, m_duContext->topContext());
+      if(!t)
+        return IndexedType();
+    }
+  }
+  
+  return t->indexed();
+}
+
 CodeCompletionContext::~CodeCompletionContext() {
 }
 
@@ -663,8 +771,13 @@ void CodeCompletionContext::preprocessText( int line ) {
 
   LOCKDUCHAIN;
 
+  QSet<IndexedString> disableMacros;
+  disableMacros.insert(IndexedString("SIGNAL"));
+  disableMacros.insert(IndexedString("SLOT"));
+  disableMacros.insert(IndexedString("emit"));
+  
   if( m_duContext ) {
-  m_text = preprocess( m_text,  dynamic_cast<Cpp::EnvironmentFile*>(m_duContext->topContext()->parsingEnvironmentFile().data()), line );
+  m_text = preprocess( m_text,  dynamic_cast<Cpp::EnvironmentFile*>(m_duContext->topContext()->parsingEnvironmentFile().data()), line, disableMacros );
   }else{
     kWarning() << "error: no ducontext";
   }
@@ -687,7 +800,7 @@ void getOverridable(DUContext* base, DUContext* current, QMap< QPair<IndexedType
         overridable.insert(key, KDevelop::CompletionTreeItemPointer(new ImplementationHelperItem(ImplementationHelperItem::Override, DeclarationPointer(decl), completionContext)));
     }
   }
-  
+
   foreach(const DUContext::Import &import, current->importedParentContexts())
     getOverridable(base, import.context(base->topContext()), overridable, completionContext);
 }
@@ -704,7 +817,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
     typedef QPair<Declaration*, int> DeclarationDepthPair;
 
     bool ignoreParentContext = false;
-    
+
     if(!m_storedItems.isEmpty()) {
       items = m_storedItems;
     }else{
@@ -716,6 +829,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
           if( memberAccessContainer().isValid() ||memberAccessOperation() == Cpp::CodeCompletionContext::StaticMemberChoose )
           {
             QList<DUContext*> containers = memberAccessContainers();
+            ifDebug( kDebug() << "got" << containers.size() << "member-access containers"; )
             if( !containers.isEmpty() ) {
               QSet<DUContext*> had;
               foreach(DUContext* ctx, containers) {
@@ -728,10 +842,15 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
 
                 foreach( const DeclarationDepthPair& decl, Cpp::hideOverloadedDeclarations( ctx->allDeclarations(ctx->range().end, m_duContext->topContext(), false ) ) ) {
                   //If we have StaticMemberChoose, which means A::Bla, show only static members, except if we're within a class that derives from the container
+                  
+                  ClassMemberDeclaration* classMember = dynamic_cast<ClassMemberDeclaration*>(decl.first);
+
+                  if(classMember && !filterDeclaration(classMember))
+                    continue;
+                    
                   if(memberAccessOperation() != Cpp::CodeCompletionContext::StaticMemberChoose) {
                     if(decl.first->kind() != Declaration::Instance)
                       continue;
-                    ClassMemberDeclaration* classMember = dynamic_cast<ClassMemberDeclaration*>(decl.first);
                     if(classMember && classMember->isStatic())
                       continue; //Skip static class members when not doing static access
                     if(decl.first->abstractType().cast<EnumeratorType>())
@@ -747,7 +866,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
             } else {
               kDebug(9007) << "setContext: no container-type";
             }
-          }          
+          }
           break;
         case ReturnAccess:
           {
@@ -758,12 +877,28 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
               FunctionType::Ptr funType = functionContext->owner()->type<FunctionType>();
               if(funType) {
                 if(funType->returnType()) {
-                  items << CompletionTreeItemPointer( new TypeConversionCompletionItem( "return " + funType->returnType()->toString(), funType->returnType()->indexed(), depth() ) );
+                  items << CompletionTreeItemPointer( new TypeConversionCompletionItem( "return " + funType->returnType()->toString(), funType->returnType()->indexed(), depth(), KSharedPtr <Cpp::CodeCompletionContext >(this) ) );
                 }
               }
             }
           }
         break;
+        case TemplateAccess:
+          {
+            AbstractType::Ptr type = m_expressionResult.type.type();
+            IdentifiedType* identified = dynamic_cast<IdentifiedType*>(type.unsafeData());
+            Declaration* decl = 0;
+            if(identified)
+              decl = identified->declaration( m_duContext->topContext());
+            if(!decl && !m_expressionResult.allDeclarations.isEmpty())
+              decl = m_expressionResult.allDeclarations[0].getDeclaration(m_duContext->topContext());
+            if(decl) {
+              NormalDeclarationCompletionItem* item = new NormalDeclarationCompletionItem( KDevelop::DeclarationPointer(decl),  KSharedPtr <Cpp::CodeCompletionContext >(this), 0, 0 );
+              item->m_isTemplateCompletion = true;
+              items << CompletionTreeItemPointer( item );
+            }
+          }
+          break;
         case FunctionCallAccess:
           {
             //Don't show annoying empty argument-hints
@@ -783,7 +918,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
                 ++num;
               }
             }
-            
+
             if(additionalContextType() == Cpp::CodeCompletionContext::BinaryOperatorFunctionCall) {
               //Argument-hints for builtin operators
               AbstractType::Ptr type = m_expressionResult.type.type();
@@ -798,24 +933,24 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
                   if(!conv.implicitConversion(m_expressionResult.type, integral->indexed()))
                     integral = IntegralType::Ptr(); //No conversion possible
                 }
-                
+
                 if( m_operator == "=" || integral ) {
                   ///Conversion to the left operand-type, builtin operators on integral types
                   IndexedType useType = integral ? integral->indexed() : m_expressionResult.type;
                   QString showName = functionName();
                   if(useType.type())
                     showName = useType.type()->toString() + " " + m_operator;
-                  
+
                   if(useType == m_expressionResult.type && m_expressionResult.allDeclarations.size() == 1) {
                     Declaration* decl = m_expressionResult.allDeclarations[0].getDeclaration(m_duContext->topContext());
                     if(decl)
                       showName = decl->toString() + " " + m_operator;
                   }
-                  
-                  items << CompletionTreeItemPointer( new TypeConversionCompletionItem( showName, useType, depth() ) );
+
+                  items << CompletionTreeItemPointer( new TypeConversionCompletionItem( showName, useType, depth(), KSharedPtr <Cpp::CodeCompletionContext >(this) ) );
                 }
               }
-              
+
 //                 items.back()->asItem<NormalDeclarationCompletionItem>()->alternativeText = functionName();
             }
           }
@@ -834,12 +969,12 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
               ++cnt;
             }
             kDebug(9007) << "Added " << cnt << " include-files to completion-list";
-          }          
+          }
           break;
         case SignalAccess:
         case SlotAccess:
         if(!m_connectedSignalIdentifier.isEmpty()) {
-          //Create an additional argument-hint context that shows information about the signal we connect to
+          ///Create an additional argument-hint context that shows information about the signal we connect to
           if(parentContext() && parentContext()->m_knownArgumentTypes.count() > 1 && parentContext()->m_knownArgumentTypes[0].type.isValid()) {
             StructureType::Ptr signalContainerType = TypeUtils::targetType(parentContext()->m_knownArgumentTypes[0].type.type(), m_duContext->topContext()).cast<StructureType>();
            if(signalContainerType) {
@@ -866,7 +1001,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
           }
         }
         if( memberAccessContainer().isValid() ) {
-          //Collect all slots
+          ///Collect all slots/signals to show
           AbstractType::Ptr type = memberAccessContainer().type.type();
           IdentifiedType* identified = dynamic_cast<IdentifiedType*>(type.unsafeData());
           if(identified) {
@@ -874,7 +1009,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
             if(decl && decl->internalContext()) {
               foreach(const DeclarationDepthPair &candidate, decl->internalContext()->allDeclarations(SimpleCursor::invalid(), m_duContext->topContext(), false) ) {
                 if(QtFunctionDeclaration* classFun = dynamic_cast<QtFunctionDeclaration*>(candidate.first)) {
-                  if(classFun->isSignal() || (memberAccessOperation() == SlotAccess && classFun->isSlot())) {
+                  if((classFun->isSignal() && !m_onlyShowSlots) || (memberAccessOperation() == SlotAccess && classFun->isSlot() && filterDeclaration(classFun))) {
                     NormalDeclarationCompletionItem* item = new NormalDeclarationCompletionItem( DeclarationPointer(candidate.first), CodeCompletionContext::Ptr(this), candidate.second );
                     item->m_isQtSignalSlotCompletion = true;
                     if(!m_connectedSignalIdentifier.isEmpty()) {
@@ -907,11 +1042,12 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
 //         if(parentContext() && parentContext()->m_knownArgumentExpressions.size() != 2)
 //           break;
         default:
-          standardAccessCompletionItems(position, items);
+          if(depth() == 0)
+            standardAccessCompletionItems(position, items);
           break;
       }
     }
-      
+
     if(!ignoreParentContext && fullCompletion && m_parentContext && (!noMultipleBinaryOperators || m_contextType != BinaryOperatorFunctionCall || parentContext()->m_contextType != BinaryOperatorFunctionCall))
       items = parentContext()->completionItems( position, shouldAbort, fullCompletion ) + items;
 
@@ -921,12 +1057,12 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
         QString totalExpression = m_expression + "::" + m_followingText.trimmed();
         items += missingIncludeCompletionItems(totalExpression, m_followingText.trimmed() + ": ", ExpressionEvaluationResult(), m_duContext.data(), 0);
       }
-      
+
       if(items.isEmpty() && (memberAccessOperation() == StaticMemberChoose || memberAccessOperation() == MemberChoose || memberAccessOperation() == MemberAccess || memberAccessOperation() == ArrowMemberAccess)) {
         kDebug() << "doing missing-include completion for" << m_expression;
         items += missingIncludeCompletionItems(m_expression, QString(), m_expressionResult, m_duContext.data(), 0, memberAccessOperation() == StaticMemberChoose);
       }
-      
+
       if(m_duContext->type() == DUContext::Class) {
         //Show override helper items
         QMap< QPair<IndexedType, IndexedString>, KDevelop::CompletionTreeItemPointer > overridable;
@@ -934,7 +1070,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(const KD
           getOverridable(m_duContext.data(), import.context(m_duContext->topContext()), overridable, Ptr(this));
         items += overridable.values();
       }
-      
+
       if(m_duContext->type() == DUContext::Namespace || m_duContext->type() == DUContext::Global)
         items += getImplementationHelpers();
     }
@@ -949,17 +1085,17 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::getImplementationHelpers
     KUrl headerUrl = CppLanguageSupport::self()->sourceOrHeaderCandidate( searchInContext->url().toUrl(), true );
     searchInContext = CppLanguageSupport::self()->standardContext(headerUrl);
   }
-  
+
   if(searchInContext) {
     return getImplementationHelpersInternal(m_duContext->scopeIdentifier(true), searchInContext);
   }
-#endif  
+#endif
   return QList<CompletionTreeItemPointer>();
 }
 
 QList<CompletionTreeItemPointer> CodeCompletionContext::getImplementationHelpersInternal(QualifiedIdentifier minimumScope, DUContext* context) {
   QList<CompletionTreeItemPointer> ret;
-  
+
   foreach(Declaration* decl, context->localDeclarations()) {
     ClassFunctionDeclaration* classFun = dynamic_cast<ClassFunctionDeclaration*>(decl);
     if(classFun) {
@@ -967,7 +1103,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::getImplementationHelpers
         ret << KDevelop::CompletionTreeItemPointer(new ImplementationHelperItem(ImplementationHelperItem::CreateDefinition, DeclarationPointer(classFun), KSharedPtr<CodeCompletionContext>(this)));
       }
   }
-  
+
   foreach(DUContext* child, context->childContexts())
     if(child->type() == DUContext::Namespace || child->type() == DUContext::Class)
       ret += getImplementationHelpersInternal(minimumScope, child);
@@ -993,12 +1129,32 @@ QualifiedIdentifier CodeCompletionContext::requiredPrefix(Declaration* decl) con
   }
 }
 
+QList< KSharedPtr< KDevelop::CompletionTreeItem > > CodeCompletionContext::specialItemsForArgumentType(TypePtr< KDevelop::AbstractType > type) {
+  QList< KSharedPtr< KDevelop::CompletionTreeItem > > items;
+  if(EnumerationType::Ptr enumeration = TypeUtils::realType(type, m_duContext->topContext()).cast<EnumerationType>()) {
+    Declaration* enumDecl = enumeration->declaration(m_duContext->topContext());
+    if(enumDecl && enumDecl->internalContext()) {
+
+      QualifiedIdentifier prefix = requiredPrefix(enumDecl);
+
+      DUContext* enumInternal = enumDecl->internalContext();
+      foreach(Declaration* enumerator, enumInternal->localDeclarations()) {
+        QualifiedIdentifier id = prefix + enumerator->identifier();
+        items << CompletionTreeItemPointer(new NormalDeclarationCompletionItem( DeclarationPointer(enumerator), Ptr(this), 0 ));
+        static_cast<NormalDeclarationCompletionItem*>(items.back().data())->alternativeText = id.toString();
+        static_cast<NormalDeclarationCompletionItem*>(items.back().data())->useAlternativeText = true;
+      }
+    }
+  }
+  return items;
+}
+
 void CodeCompletionContext::standardAccessCompletionItems(const KDevelop::SimpleCursor& position, QList<CompletionTreeItemPointer>& items) {
   //Normal case: Show all visible declarations
   typedef QPair<Declaration*, int> DeclarationDepthPair;
 
   QList<DeclarationDepthPair> decls = m_duContext->allDeclarations(m_duContext->type() == DUContext::Class ? m_duContext->range().end : position, m_duContext->topContext());
-  
+
   //Collect the contents of unnamed namespaces
   QList<DUContext*> unnamed = m_duContext->findContexts(DUContext::Namespace, QualifiedIdentifier(), position);
   foreach(DUContext* ns, unnamed)
@@ -1034,45 +1190,49 @@ void CodeCompletionContext::standardAccessCompletionItems(const KDevelop::Simple
     if(!dynamic_cast<FunctionDefinition*>(decl.first) || !static_cast<FunctionDefinition*>(decl.first)->hasDeclaration())
       if(filterDeclaration(decl.first))
         decls << decl;
-  
+
   decls = Cpp::hideOverloadedDeclarations(decls);
 
   foreach( const DeclarationDepthPair& decl, decls )
     items << CompletionTreeItemPointer( new NormalDeclarationCompletionItem(DeclarationPointer(decl.first), Ptr(this), decl.second ) );
 
+  ///Eventually show additional specificly known items for the matched argument-type, like for example enumerators for enum types
   CodeCompletionContext* parent = parentContext();
   if(parent) {
     if(parent->memberAccessOperation() == FunctionCallAccess) {
-        foreach(const Cpp::OverloadResolutionFunction& function, parent->functions()) {
-          if(function.function.isValid() && function.function.isViable() && function.function.declaration()) {
-            //uint parameterNumber = parent->m_knownArgumentExpressions.size() + function.matchedArguments;
-            Declaration* functionDecl = function.function.declaration().data();
-            if(functionDecl->type<FunctionType>()->arguments().count() > function.matchedArguments) {
-              //Eventually pick additional specificly known items for the type
-              AbstractType::Ptr type = functionDecl->type<FunctionType>()->arguments()[function.matchedArguments];
-              if(type) {
-                if(EnumerationType::Ptr enumeration = TypeUtils::realType(type, m_duContext->topContext()).cast<EnumerationType>()) {
-                  Declaration* enumDecl = enumeration->declaration(m_duContext->topContext());
-                  if(enumDecl && enumDecl->internalContext()) {
-
-                    QualifiedIdentifier prefix = requiredPrefix(enumDecl);
-
-                    DUContext* enumInternal = enumDecl->internalContext();
-                    foreach(Declaration* enumerator, enumInternal->localDeclarations()) {
-                      QualifiedIdentifier id = prefix + enumerator->identifier();
-                      items << CompletionTreeItemPointer(new NormalDeclarationCompletionItem( DeclarationPointer(enumerator), Ptr(this), 0 ));
-                      static_cast<NormalDeclarationCompletionItem*>(items.back().data())->alternativeText = id.toString();
-                      static_cast<NormalDeclarationCompletionItem*>(items.back().data())->useAlternativeText = true;
-                    }
-                  }
-                }
-              }
-            }
+      foreach(const Cpp::OverloadResolutionFunction& function, parent->functions()) {
+        if(function.function.isValid() && function.function.isViable() && function.function.declaration()) {
+          //uint parameterNumber = parent->m_knownArgumentExpressions.size() + function.matchedArguments;
+          Declaration* functionDecl = function.function.declaration().data();
+          if(functionDecl->type<FunctionType>()->arguments().count() > function.matchedArguments) {
+            items += specialItemsForArgumentType(functionDecl->type<FunctionType>()->arguments()[function.matchedArguments]);
           }
+        }
       }
     }
   }
 
+  ///Eventually add a "this" item
+  DUContext* functionContext = m_duContext.data();
+  if(!m_onlyShowSignals && !m_onlyShowSlots && !m_onlyShowTypes) {
+    while(functionContext && functionContext->type() == DUContext::Other && functionContext->parentContext()->type() == DUContext::Other)
+      functionContext = functionContext->parentContext();
+  }
+
+  ClassFunctionDeclaration* classFun = dynamic_cast<ClassFunctionDeclaration*>(DUChainUtils::declarationForDefinition(functionContext->owner(), m_duContext->topContext()));
+  
+  if(classFun && !classFun->isStatic() && classFun->context()->owner()) {
+    AbstractType::Ptr classType = classFun->context()->owner()->abstractType();
+    if(classFun->abstractType()->modifiers() & AbstractType::ConstModifier)
+      classType->setModifiers((AbstractType::CommonModifiers)(classType->modifiers() | AbstractType::ConstModifier));
+    PointerType::Ptr thisPointer(new PointerType());
+    thisPointer->setModifiers(AbstractType::ConstModifier);
+    thisPointer->setBaseType(classType);
+    KSharedPtr<TypeConversionCompletionItem> item( new TypeConversionCompletionItem("this", thisPointer->indexed(), 0, KSharedPtr <Cpp::CodeCompletionContext >(this)) );
+    item->setPrefix(thisPointer->toString());
+    items += CompletionTreeItemPointer(item.data());
+  }
+
   //Eventually add missing include-completion in cases like NotIncludedClass|
   if(!m_followingText.trimmed().isEmpty()) {
     QString totalExpression = m_followingText.trimmed();
@@ -1082,12 +1242,37 @@ void CodeCompletionContext::standardAccessCompletionItems(const KDevelop::Simple
   }
 }
 
-bool  CodeCompletionContext::filterDeclaration(Declaration* decl) {
-  if(m_onlyShowTypes)
-    return decl->kind() == Declaration::Type; //Only show type declarations within class contexts
+bool  CodeCompletionContext::filterDeclaration(Declaration* decl, bool dynamic) {
+  if(!decl)
+    return true;
+  
+  if(m_onlyShowTypes && decl->kind() != Declaration::Type)
+    return false;
+    
+  if(m_onlyShowSignals || m_onlyShowSlots) {
+    Cpp::QtFunctionDeclaration* qtFunction = dynamic_cast<Cpp::QtFunctionDeclaration*>(decl);
+    if(!qtFunction || (m_onlyShowSignals && !qtFunction->isSignal()) || (m_onlyShowSlots && !qtFunction->isSlot()))
+      return false;
+  }
+  
+  if(dynamic && decl->context()->type() == DUContext::Class) {
+    ClassMemberDeclaration* classMember = dynamic_cast<ClassMemberDeclaration*>(decl);
+    if(classMember)
+      return filterDeclaration(classMember);
+  }
+  
   return true;
 }
 
+bool  CodeCompletionContext::filterDeclaration(ClassMemberDeclaration* decl) {
+    
+  if(m_localClass && doAccessFiltering && decl) {
+    if(!Cpp::isAccessible(m_localClass.data(), decl))
+      return false;
+  }
+  return filterDeclaration((Declaration*)decl, false);
+}
+
 void CodeCompletionContext::replaceCurrentAccess(QString old, QString _new)
 {
   IDocument* document = ICore::self()->documentController()->documentForUrl(m_duContext->url().toUrl());
@@ -1106,5 +1291,9 @@ void CodeCompletionContext::replaceCurrentAccess(QString old, QString _new)
   }
 }
 
+int CodeCompletionContext::matchPosition() const {
+  return m_knownArgumentExpressions.count();
+}
+
 
 }
