@@ -145,9 +145,15 @@ void addImportedParentContextSafely(DUContext* context, DUContext* import) {
   }
 }
 
-KDevelop::QualifiedIdentifier ContextBuilder::identifierForNode(NameAST* id)
+QualifiedIdentifier ContextBuilder::identifierForNode(NameAST* id) {
+  QualifiedIdentifier ret;
+  identifierForNode(id, ret);
+  return ret;
+}
+
+void ContextBuilder::identifierForNode(NameAST* id, QualifiedIdentifier& target)
 {
-  return identifierForNode(id, 0);
+  return identifierForNode(id, 0, target);
 }
 
 void ContextBuilder::startVisiting( AST* node )
@@ -192,7 +198,9 @@ void ContextBuilder::openPrefixContext(ClassSpecifierAST* ast, const QualifiedId
   {
     DUChainReadLocker lock(DUChain::lock());
 
-    QualifiedIdentifier globalId = currentContext()->scopeIdentifier(true);
+    QualifiedIdentifier currentScopeId = currentContext()->scopeIdentifier(true);
+    
+    QualifiedIdentifier globalId = currentScopeId;
     globalId += prefixId;
     globalId.setExplicitlyGlobal(true);
 
@@ -200,8 +208,16 @@ void ContextBuilder::openPrefixContext(ClassSpecifierAST* ast, const QualifiedId
 
     if(!decls.isEmpty()) {
       DUContext* classContext = decls.first()->logicalInternalContext(0);
-      if(classContext && classContext->type() == DUContext::Class)
+      if(classContext && classContext->type() == DUContext::Class) {
         import = classContext;
+        //Change the prefix-id so it respects namespace-imports
+        
+        prefixId = classContext->scopeIdentifier(true);
+        if(prefixId.count() >= currentScopeId.count())
+          prefixId = prefixId.mid(currentScopeId.count());
+        else
+          kDebug() << "resolved bad prefix context. Should start with" << currentScopeId.toString() << "but is" << prefixId.toString();
+      }
     }
   }
 
@@ -303,9 +319,12 @@ ReferencedTopDUContext ContextBuilder::buildContexts(const Cpp::EnvironmentFileP
   Q_ASSERT(file);
   setCompilingContexts(true);
 
-  if(updateContext && (updateContext->parsingEnvironmentFile() && updateContext->parsingEnvironmentFile()->isProxyContext())) {
-    kDebug(9007) << "updating a context " << file->url().str() << " from a proxy-context to a content-context";
-    updateContext->parsingEnvironmentFile()->setIsProxyContext(false);
+  {
+    DUChainWriteLocker lock(DUChain::lock());
+    if(updateContext && (updateContext->parsingEnvironmentFile() && updateContext->parsingEnvironmentFile()->isProxyContext())) {
+      kDebug(9007) << "updating a context " << file->url().str() << " from a proxy-context to a content-context";
+      updateContext->parsingEnvironmentFile()->setIsProxyContext(false);
+    }
   }
 
   if(editor()->currentUrl() != file->url())
@@ -494,6 +513,10 @@ void ContextBuilder::visitEnumSpecifier(EnumSpecifierAST* node)
   closeContext();
 }
 
+void ContextBuilder::classContextOpened(ClassSpecifierAST *node, DUContext* context)
+{
+}
+
 void ContextBuilder::visitClassSpecifier (ClassSpecifierAST *node)
 {
   //We only use the local identifier here, because we build a prefix-context around
@@ -518,6 +541,8 @@ void ContextBuilder::visitClassSpecifier (ClassSpecifierAST *node)
       currentContext()->setPropagateDeclarations(true);
     }
   }
+  
+  classContextOpened(node, currentContext());
 
   DefaultVisitor::visitClassSpecifier (node);
 
@@ -538,17 +563,26 @@ void ContextBuilder::visitFunctionDefinition (FunctionDefinitionAST *node)
 
   QualifiedIdentifier functionName;
   if (compilingContexts() && node->init_declarator && node->init_declarator->declarator && node->init_declarator->declarator->id) {
-    functionName = identifierForNode(node->init_declarator->declarator->id);
+    identifierForNode(node->init_declarator->declarator->id, functionName);
     if (functionName.count() >= 2) {
+      
       // This is a class function definition
       DUChainReadLocker lock(DUChain::lock());
-      QualifiedIdentifier className = currentContext()->scopeIdentifier(false) + functionName;
+      QualifiedIdentifier currentScope = currentContext()->scopeIdentifier(true);
+      QualifiedIdentifier className = currentScope + functionName;
       className.pop();
       className.setExplicitlyGlobal(true);
-
+      
       QList<Declaration*> classDeclarations = currentContext()->findDeclarations(className);
-      if (classDeclarations.count() != 0 && classDeclarations.first()->internalContext())
+      
+      if (classDeclarations.count() != 0 && classDeclarations.first()->internalContext()) {
         m_importedParentContexts.append(classDeclarations.first()->internalContext());
+        
+        QualifiedIdentifier newFunctionName(className);
+        newFunctionName.push(functionName.last());
+        if(newFunctionName.count() > currentScope.count())
+          functionName = newFunctionName.mid(currentScope.count());
+      }
     }
   }
   visitFunctionDeclaration(node);
@@ -729,7 +763,9 @@ public:
   virtual void visitName (NameAST * node)
   {
     if (result) {
-      if (!builder->currentContext()->findDeclarations(builder->identifierForNode(node), cursor).isEmpty()) {
+      QualifiedIdentifier id;
+      builder->identifierForNode(node, id);
+      if (!builder->currentContext()->findDeclarations(id, cursor).isEmpty()) {
         result = false;
       }else{
       }
@@ -952,13 +988,19 @@ void ContextBuilder::visitTryBlockStatement(TryBlockStatementAST *node)
 {
   QList<DUContext*> parentContextsToImport = m_importedParentContexts;
 
-  openContext(node->try_block, DUContext::Other, m_openingFunctionBody);
-  m_openingFunctionBody.clear();
-  addImportedContexts();
+  if(node->try_block->kind != AST::Kind_CompoundStatement) {
+    openContext(node->try_block, DUContext::Other, m_openingFunctionBody);
+    m_openingFunctionBody.clear();
+    addImportedContexts();
 
-  visit(node->try_block);
+    visit(node->try_block);
 
-  closeContext();
+    closeContext();
+  }else{
+    //Do not double-open a context on the same node, because that will lead to problems in the mapping
+    //and failures in use-building
+    visit(node->try_block);
+  }
 
   m_tryParentContexts.push(parentContextsToImport);
 
@@ -1009,15 +1051,15 @@ bool ContextBuilder::createContextIfNeeded(AST* node, const QList<DUContext*>& i
   return contextNeeded;
 }
 
-QualifiedIdentifier ContextBuilder::identifierForNode(NameAST* id, TypeSpecifierAST** typeSpecifier)
+void ContextBuilder::identifierForNode(NameAST* id, TypeSpecifierAST** typeSpecifier, QualifiedIdentifier& target)
 {
-  if( !id )
-    return QualifiedIdentifier();
+  if( !id ) {
+    target = QualifiedIdentifier();
+  }
 
-  m_nameCompiler->run(id);
+  m_nameCompiler->run(id, &target);
   if( typeSpecifier )
     *typeSpecifier = m_nameCompiler->lastTypeSpecifier();
-  return m_nameCompiler->identifier();
 }
 
 bool ContextBuilder::smart() const {
