@@ -82,18 +82,18 @@ extern QMutex cppDuContextInstantiationsMutex;
         void openQualifiedIdentifier( bool isExplicitlyGlobal ) {
           QualifiedIdentifier i;
           i.setExplicitlyGlobal( isExplicitlyGlobal );
-          State s;
-          s.identifier = i;
+          StatePtr s(new State);
+          s->identifier = i;
           m_states << s;
         }
 
         ///Can be used to just append a result that was computed outside. closeQualifiedIdentifier(...) must still be called.
         void openQualifiedIdentifier( const ExpressionEvaluationResult& result ) {
-          State s;
-          s.expressionResult = result;
-          s.result.clear();
+          StatePtr s(new State);
+          s->expressionResult = result;
+          s->result.clear();
           foreach(const DeclarationId& decl, result.allDeclarations)
-          s.result << DeclarationPointer( decl.getDeclaration(const_cast<TopDUContext*>(topContext())) );
+          s->result << DeclarationPointer( decl.getDeclaration(const_cast<TopDUContext*>(topContext())) );
           
           m_states << s;
         }
@@ -107,7 +107,7 @@ extern QMutex cppDuContextInstantiationsMutex;
          * The identifier must not have template identifiers, those need to be added using openQualifiedIdentifier(..) and closeQualifiedIdentifier(..)
          * */
         void openIdentifier( const Identifier& identifier ) {
-         m_states.top().identifier.push(identifier);
+         m_states.top()->identifier.push(identifier);
         }
         /**
          * When closeIdentifier() is called, the last opened identifier is searched, and can be retrieved using lastDeclarations().
@@ -137,13 +137,17 @@ extern QMutex cppDuContextInstantiationsMutex;
         const TopDUContext* topContext() const {
           return m_source;
         }
+        
+        uint currentLength() const {
+          return m_states.size();
+        }
 
       private:
 
         ///Uses the instantiation-information from the context of decl as parent of templateArguments.
         Declaration* instantiateDeclaration( Declaration* decl, const InstantiationInformation& templateArguments ) const;
         
-        struct State {
+        struct State : public KShared {
           State() {
           }
           QualifiedIdentifier identifier; //identifier including eventual namespace prefix
@@ -153,7 +157,10 @@ extern QMutex cppDuContextInstantiationsMutex;
           QList<DeclarationPointer> result;
           ExpressionEvaluationResult expressionResult;
         };
-        QStack<State> m_states;
+        
+        typedef KSharedPtr<State> StatePtr;
+        
+        QStack<StatePtr> m_states;
         const DUContext* m_context;
         const TopDUContext* m_source;
         DUContext::SearchFlags m_flags;
@@ -194,6 +201,37 @@ class CppDUContext : public BaseContext {
     ///Overridden to take care of templates and other c++ specific things
     virtual bool findDeclarationsInternal(const DUContext::SearchItem::PtrList& identifiers, const SimpleCursor& position, const AbstractType::Ptr& dataType, DUContext::DeclarationList& ret, const TopDUContext* source, typename BaseContext::SearchFlags basicFlags ) const
     {
+      if(this->type() == DUContext::Class && identifiers.count() == 1 && !identifiers[0]->hasNext() && !(basicFlags & DUContext::OnlyFunctions) && this->localScopeIdentifier().count()) {
+        
+        //Check whether we're searching for just the name of this context's class. If yes, return this context's owner.
+        Identifier searchId = identifiers[0]->identifier;
+        if(searchId.identifier() == this->localScopeIdentifier().last().identifier() && !searchId.templateIdentifiersCount()) {
+          
+          Declaration* owner = this->owner();
+          if(owner) {
+            if(basicFlags & DUContext::NoUndefinedTemplateParams) {
+              //If no undefined template parameters are allowed, make sure this template has all parameters assigned.
+              TemplateDeclaration* templateOwner = dynamic_cast<TemplateDeclaration*>(this->owner());
+              if(templateOwner) {
+                if(!templateOwner->instantiatedFrom())
+                  return false;
+                DUContext* templateContext = templateOwner->templateContext(source);
+                if(templateContext) {
+                  foreach(Declaration* decl, templateContext->localDeclarations()) {
+                    if(decl->type<CppTemplateParameterType>()) {
+                      return false;
+                    }
+                  }
+                }
+              }
+            }
+            
+            ret << this->owner();
+            return true;
+          }
+        }
+      }
+      
       if( basicFlags & BaseContext::DirectQualifiedLookup ) {
         //ifDebug( kDebug(9007) << "redirecting findDeclarationsInternal in " << this << "(" << this->scopeIdentifier() <<") for \"" << identifier.toString() << "\""; )
         //We use DirectQualifiedLookup to signalize that we don't need to do the whole scope-search, template-resolution etc. logic.
@@ -356,8 +394,9 @@ class CppDUContext : public BaseContext {
               }else{
                 copy = templateDecl->instantiate(memberInstantiationInformation, source);
               }
-              if(copy->context() != this)
-                kWarning() << "serious problem: Instatiation is in wrong context, should be in this one";
+              //This can happen in case of explicit specializations
+//               if(copy->context() != this)
+//                 kWarning() << "serious problem: Instatiation is in wrong context, should be in this one";
 
               ret.append(copy);
             }
@@ -369,6 +408,7 @@ class CppDUContext : public BaseContext {
       QMutexLocker l(&cppDuContextInstantiationsMutex);
       foreach(CppDUContext<BaseContext>* ctx, m_instatiations)
         ctx->deleteUses();
+      BaseContext::deleteUses();
     }
     
     virtual bool foundEnough( const DUContext::DeclarationList& decls, DUContext::SearchFlags flags ) const
@@ -445,24 +485,34 @@ class CppDUContext : public BaseContext {
       {
         QualifiedIdentifier prefix = BaseContext::localScopeIdentifier();
         if(prefix.count() > 1) {
-          //This must be a function-definition, like void A::B::test() {}
-          KDevelop::DUContext* classContext  = 0;
+          
+          DUContext* classContext = 0;
           
           if(type == DUContext::Helper) {
-            if(!BaseContext::importedParentContexts().isEmpty())
+            //This is a prefix-context for an external class-definition like "class A::B {..};"
+            if(BaseContext::importedParentContexts().size())
               classContext = BaseContext::importedParentContexts()[0].context(source);
           } else {
+            //This must be a function-definition, like void A::B::test() {}
             Declaration* classDeclaration = Cpp::localClassFromCodeContext(const_cast<BaseContext*>((const BaseContext*)this));
-            if(classDeclaration && classDeclaration->internalContext())
-              classContext = classDeclaration->internalContext();
+            if(classDeclaration)
+              classContext = classDeclaration->logicalInternalContext(source);
+            prefix.pop();
           }
           
           if(classContext) {
-            //If this is a definition of a class member, only add aliases for the namespace elements(The class scope will be
-            //searched using then normal import logic)
-            prefix = classContext->scopeIdentifier(false);
-          }else{
-            prefix.pop();
+            while(!prefix.isEmpty() && classContext && classContext->type() == DUContext::Class) {
+              prefix.pop();
+              
+              //This way we can correctly resolve the namespace-component for multiple externally defined classes,
+              //see testDeclareStructInNamespace() in test_duchain.cpp
+              if(classContext->parentContext() && classContext->parentContext()->type() == DUContext::Helper) {
+                classContext = BaseContext::importedParentContexts()[0].context(source);
+                continue;
+              }
+              
+              break;
+            }
           }
           
           if(!prefix.isEmpty()) {
@@ -496,7 +546,7 @@ class CppDUContext : public BaseContext {
     
     virtual bool shouldSearchInParent(typename BaseContext::SearchFlags flags) const {
       //If the parent context is a class context, we should even search it from an import
-      return !(flags & DUContext::InImportedParentContext) || (BaseContext::parentContext() && BaseContext::parentContext()->type() == DUContext::Class);
+      return (BaseContext::parentContext() && BaseContext::parentContext()->type() == DUContext::Class) || BaseContext::shouldSearchInParent(flags);
     }
 
     virtual DUContext* specialize(uint specialization, const TopDUContext* topContext, int upDistance) {
@@ -571,7 +621,10 @@ class CppDUContext : public BaseContext {
       }
       foreach( CppDUContext<BaseContext>* instatiation, instatiations ) {
         Q_ASSERT(instatiation != this);
-        delete instatiation;
+        if(instatiation->isAnonymous())
+          delete instatiation;
+        else
+          instatiation->setInstantiatedFrom(0, InstantiationInformation());
       }
     }
 
