@@ -57,6 +57,7 @@
 #include "classdeclaration.h"
 
 #include "cppdebughelper.h"
+#include "name_visitor.h"
 
 const Identifier& castIdentifier() {
   static Identifier id("operator{...cast...}");
@@ -133,10 +134,10 @@ ReferencedTopDUContext DeclarationBuilder::buildDeclarations(const Cpp::Environm
 void DeclarationBuilder::visitTemplateParameter(TemplateParameterAST * ast) {
   
   //Backup and zero the parameter declaration, because we will handle it here directly, and don't want a normal one to be created
-  ParameterDeclarationAST* paramAST = ast->parameter_declaration;
-  ast->parameter_declaration = 0;
+  
+  m_ignoreDeclarators = true;
   DeclarationBuilderBase::visitTemplateParameter(ast);
-  ast->parameter_declaration = paramAST;
+  m_ignoreDeclarators = false;
   
   if( ast->type_parameter || ast->parameter_declaration ) {
     ///@todo deal with all the other stuff the AST may contain
@@ -261,7 +262,11 @@ void DeclarationBuilder::visitDeclarator (DeclaratorAST* node)
 
       DUChainWriteLocker lock(DUChain::lock());
       //We have to search for the fully qualified identifier, so we always get the correct class
-      QualifiedIdentifier id = currentContext()->scopeIdentifier(false) + identifierForNode(node->id);
+      QualifiedIdentifier id = currentContext()->scopeIdentifier(false);
+      QualifiedIdentifier id2;
+      identifierForNode(node->id, id2);
+      id += id2;
+      
       id.setExplicitlyGlobal(true);
 
       if (id.count() > 1 ||
@@ -399,7 +404,8 @@ T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const
     if(name->unqualified_name && name->unqualified_name->operator_id)
       visit(name->unqualified_name->operator_id);
     
-    QualifiedIdentifier id = identifierForNode(name);
+    QualifiedIdentifier id;
+    identifierForNode(name, id);
 
     if(localId.isEmpty())
       localId = id.last();
@@ -443,6 +449,10 @@ T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const
          )
       {
         // Match
+        TemplateDeclaration* templateDecl = dynamic_cast<TemplateDeclaration*>(dec);
+        if(templateDecl)
+          templateDecl->deleteAllInstantiations(); //Delete all instantiations so we have a fresh start
+        
         declaration = dynamic_cast<T*>(dec);
         break;
       }
@@ -500,7 +510,7 @@ T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const
 
     for(int a = backup.size()-1; a >= 0; --a)
       editor()->setCurrentRange(iface, backup[a]);
-
+  
     Q_ASSERT(editor()->currentRange(iface) == prior);
 
     declaration = new T(newRange, currentContext());
@@ -534,7 +544,7 @@ T* DeclarationBuilder::openDeclarationReal(NameAST* name, AST* rangeNode, const
             break;
           }
 
-        if( !templateDecl->specializedFrom() )
+        if( !templateDecl->specializedFrom().isValid() )
           kDebug(9007) << "Could not find valid specialization-base" << localId.toString() << "for" << declaration->toString();
       }
     } else {
@@ -595,7 +605,8 @@ Declaration* DeclarationBuilder::openNormalDeclaration(NameAST* name, AST* range
 
 Declaration* DeclarationBuilder::openFunctionDeclaration(NameAST* name, AST* rangeNode) {
 
-   QualifiedIdentifier id = identifierForNode(name);
+   QualifiedIdentifier id;
+   identifierForNode(name, id);
    Identifier localId = id.last(); //This also copies the template arguments
    if(id.count() > 1) {
      //Merge the scope of the declaration, and put them tog. Add semicolons instead of the ::, so you can see it's not a qualified identifier.
@@ -659,32 +670,43 @@ void DeclarationBuilder::classTypeOpened(AbstractType::Ptr type) {
 
 void DeclarationBuilder::closeDeclaration(bool forceInstance)
 {
-  if (lastType()) {
+  {
     DUChainWriteLocker lock(DUChain::lock());
+      
+    if (lastType()) {
 
-    AbstractType::Ptr type = lastType();
-    IdentifiedType* idType = dynamic_cast<IdentifiedType*>(type.unsafeData());
-    DelayedType::Ptr delayed = type.cast<DelayedType>();
+      AbstractType::Ptr type = lastType();
+      IdentifiedType* idType = dynamic_cast<IdentifiedType*>(type.unsafeData());
+      DelayedType::Ptr delayed = type.cast<DelayedType>();
 
-    //When the given type has no declaration yet, assume we are declaring it now.
-    //If the type is a delayed type, it is a searched type, and not a declared one, so don't set the declaration then.
-    if( !forceInstance && idType && !idType->declarationId().isValid() && !delayed ) {
-        idType->setDeclaration( currentDeclaration() );
-        //Q_ASSERT(idType->declaration() == currentDeclaration());
+      //When the given type has no declaration yet, assume we are declaring it now.
+      //If the type is a delayed type, it is a searched type, and not a declared one, so don't set the declaration then.
+      if( !forceInstance && idType && !idType->declarationId().isValid() && !delayed ) {
+          idType->setDeclaration( currentDeclaration() );
+          //Q_ASSERT(idType->declaration() == currentDeclaration());
+      }
+
+      if(currentDeclaration()->kind() != Declaration::NamespaceAlias && currentDeclaration()->kind() != Declaration::Alias) {
+        //If the type is not identified, it is an instance-declaration too, because those types have no type-declarations.
+        if( (((!idType) || (idType && idType->declarationId() != currentDeclaration()->id())) && !currentDeclaration()->isTypeAlias() && !currentDeclaration()->isForwardDeclaration() ) || dynamic_cast<AbstractFunctionDeclaration*>(currentDeclaration()) || forceInstance )
+          currentDeclaration()->setKind(Declaration::Instance);
+        else
+          currentDeclaration()->setKind(Declaration::Type);
+      }
+
+      currentDeclaration()->setType(lastType());
+    }else{
+      currentDeclaration()->setAbstractType(AbstractType::Ptr());
     }
-
-    if(currentDeclaration()->kind() != Declaration::NamespaceAlias && currentDeclaration()->kind() != Declaration::Alias) {
-      //If the type is not identified, it is an instance-declaration too, because those types have no type-declarations.
-      if( (((!idType) || (idType && idType->declarationId() != currentDeclaration()->id())) && !currentDeclaration()->isTypeAlias() && !currentDeclaration()->isForwardDeclaration() ) || dynamic_cast<AbstractFunctionDeclaration*>(currentDeclaration()) || forceInstance )
-        currentDeclaration()->setKind(Declaration::Instance);
+    if(TemplateDeclaration* templateDecl = dynamic_cast<TemplateDeclaration*>(currentDeclaration())) {
+      //The context etc. may have been filled with new items, and the declaration may have been searched unsuccessfully, or wrong instantiations created.
+      if(templateDecl->instantiatedFrom())
+        templateDecl->instantiatedFrom()->deleteAllInstantiations();
+      else if(templateDecl->specializedFrom().data())
+        dynamic_cast<TemplateDeclaration*>(templateDecl->specializedFrom().data())->deleteAllInstantiations();
       else
-        currentDeclaration()->setKind(Declaration::Type);
+        templateDecl->deleteAllInstantiations();
     }
-
-    currentDeclaration()->setType(lastType());
-  }else{
-    DUChainWriteLocker lock(DUChain::lock());
-    currentDeclaration()->setAbstractType(AbstractType::Ptr());
   }
 
   eventuallyAssignInternalContext();
@@ -712,6 +734,85 @@ void DeclarationBuilder::visitEnumSpecifier(EnumSpecifierAST* node)
   closeDeclaration();
 }
 
+///Replaces a CppTemplateParameterType with a DelayedType
+struct TemplateTypeExchanger : public KDevelop::TypeExchanger {
+
+  TemplateTypeExchanger(TopDUContext* top) : m_top(top) {
+  }
+
+  virtual AbstractType::Ptr exchange( const AbstractType::Ptr& type )
+  {
+    if(CppTemplateParameterType::Ptr templateParamType = type.cast<CppTemplateParameterType>()) {
+      Declaration* decl = templateParamType->declaration(m_top);
+      if(decl) {
+        DelayedType::Ptr newType(new DelayedType());
+        
+        TypeIdentifier id(QualifiedIdentifier(decl->identifier()));
+        
+        if(type->modifiers() & AbstractType::ConstModifier)
+            id.setIsConstant(true);
+           
+        newType->setIdentifier(id);
+        newType->setKind(KDevelop::DelayedType::Delayed);
+        
+        return newType.cast<AbstractType>();
+      }
+    }
+    return type;
+  }
+  private:
+    TopDUContext* m_top;
+};
+
+Cpp::InstantiationInformation DeclarationBuilder::createSpecializationInformation(Cpp::InstantiationInformation base, UnqualifiedNameAST* name, KDevelop::DUContext* templateContext) {
+    if(name->template_arguments || base.isValid()) 
+    {
+      //Append a scope part
+      InstantiationInformation newCurrent;
+      newCurrent.previousInstantiationInformation = base.indexed().index();
+      if(!name->template_arguments)
+        return newCurrent;
+      //Process the template arguments if they exist
+      const ListNode<TemplateArgumentAST*> * start = name->template_arguments->toFront();
+      const ListNode<TemplateArgumentAST*> * current = start;
+      do {
+        NameASTVisitor visitor(editor()->parseSession(), 0, templateContext, currentContext()->topContext(), templateContext->range().end/*, DUContext::NoUndefinedTemplateParams*/);
+        ExpressionEvaluationResult res = visitor.processTemplateArgument(current->element);
+        AbstractType::Ptr type = res.type.type();
+        
+        TemplateTypeExchanger exchanger(currentContext()->topContext());
+        
+        if(type) {
+          type = exchanger.exchange(type);
+          type->exchangeTypes(&exchanger);
+        }
+        
+        newCurrent.templateParametersList().append(type->indexed());
+
+        current = current->next;
+      }while(current != start);
+      return newCurrent;
+    }else{
+      return base;
+    }
+}
+
+Cpp::IndexedInstantiationInformation DeclarationBuilder::createSpecializationInformation(NameAST* name, DUContext* templateContext)
+{
+  InstantiationInformation currentInfo;
+  if(name->qualified_names) {
+    const ListNode<UnqualifiedNameAST*> * start = name->qualified_names->toFront();
+    const ListNode<UnqualifiedNameAST*> * current = start;
+    do {
+      currentInfo = createSpecializationInformation(currentInfo, current->element, templateContext);
+      current = current->next;
+    }while(current != start);
+  }
+  if(name->unqualified_name)
+    currentInfo = createSpecializationInformation(currentInfo, name->unqualified_name, templateContext);
+  return currentInfo.indexed();
+}
+
 void DeclarationBuilder::visitEnumerator(EnumeratorAST* node)
 {
   //Ugly hack: Since we want the identifier only to have the range of the id(not
@@ -745,6 +846,13 @@ void DeclarationBuilder::visitEnumerator(EnumeratorAST* node)
   }
 }
 
+void DeclarationBuilder::classContextOpened(ClassSpecifierAST *node, DUContext* context) {
+  
+  //We need to set this early, so we can do correct search while building
+  DUChainWriteLocker lock(DUChain::lock());
+  currentDeclaration()->setInternalContext(context);
+}
+
 void DeclarationBuilder::visitClassSpecifier(ClassSpecifierAST *node)
 {
   PushValue<bool> setNotInTypedef(m_inTypedef, false);
@@ -756,10 +864,16 @@ void DeclarationBuilder::visitClassSpecifier(ClassSpecifierAST *node)
 
   SimpleCursor pos = editor()->findPosition(node->start_token, KDevelop::EditorIntegrator::FrontEdge);
 
+  IndexedInstantiationInformation specializedWith;
+  
   QualifiedIdentifier id;
   if( node->name ) {
-    id = identifierForNode(node->name);
+    identifierForNode(node->name, id);
     openPrefixContext(node, id, pos);
+    DUChainReadLocker lock(DUChain::lock());
+    if(DUContext* templateContext = hasTemplateContext(m_importedParentContexts)) {
+      specializedWith = createSpecializationInformation(node->name, templateContext);
+    }
   }
 
   int kind = editor()->parseSession()->token_stream->kind(node->class_key);
@@ -846,8 +960,14 @@ void DeclarationBuilder::visitClassSpecifier(ClassSpecifierAST *node)
 
   }//node-name
 
+  TemplateDeclaration* tempDecl = dynamic_cast<TemplateDeclaration*>(currentDeclaration());
+  
+  if(tempDecl) {
+    DUChainWriteLocker lock(DUChain::lock());
+    tempDecl->setSpecializedWith(specializedWith);
+  }
   closeDeclaration();
-
+  
   if(node->name)
     closePrefixContext(id);
 
@@ -903,6 +1023,8 @@ QualifiedIdentifier DeclarationBuilder::resolveNamespaceIdentifier(const Qualifi
     return ret;
   } else {
     QualifiedIdentifier ret = contexts.first()->scopeIdentifier(true);
+    if(ret.isEmpty())
+        return ret;
     Q_ASSERT(ret.count());
     ret.setExplicitlyGlobal(true);
     return ret;
@@ -913,7 +1035,8 @@ void DeclarationBuilder::visitUsing(UsingAST * node)
 {
   DeclarationBuilderBase::visitUsing(node);
 
-  QualifiedIdentifier id = identifierForNode(node->name);
+  QualifiedIdentifier id;
+  identifierForNode(node->name, id);
 
   ///@todo only use the last name component as range
   AliasDeclaration* decl = openDeclaration<AliasDeclaration>(0, node->name ? (AST*)node->name : (AST*)node, id.last());
@@ -925,7 +1048,7 @@ void DeclarationBuilder::visitUsing(UsingAST * node)
     if(!declarations.isEmpty()) {
       decl->setAliasedDeclaration(declarations[0]);
     }else{
-      kDebug() << "Aliased declaration not found:" << id.toString();
+      kDebug(9007) << "Aliased declaration not found:" << id.toString();
     }
   }
 
@@ -949,7 +1072,9 @@ void DeclarationBuilder::visitUsingDirective(UsingDirectiveAST * node)
     NamespaceAliasDeclaration* decl = openDeclaration<NamespaceAliasDeclaration>(0, node, globalImportIdentifier);
     {
       DUChainWriteLocker lock(DUChain::lock());
-      decl->setImportIdentifier( resolveNamespaceIdentifier(identifierForNode(node->name), currentDeclaration()->range().start) );
+      QualifiedIdentifier id;
+      identifierForNode(node->name, id);
+      decl->setImportIdentifier( resolveNamespaceIdentifier(id, currentDeclaration()->range().start) );
     }
     closeDeclaration();
   }
@@ -979,7 +1104,9 @@ void DeclarationBuilder::visitNamespaceAliasDefinition(NamespaceAliasDefinitionA
     NamespaceAliasDeclaration* decl = openDeclaration<NamespaceAliasDeclaration>(0, node, Identifier(editor()->parseSession()->token_stream->token(node->namespace_name).symbol()));
     {
       DUChainWriteLocker lock(DUChain::lock());
-      decl->setImportIdentifier( resolveNamespaceIdentifier(identifierForNode(node->alias_name), currentDeclaration()->range().start) );
+      QualifiedIdentifier id;
+      identifierForNode(node->alias_name, id);
+      decl->setImportIdentifier( resolveNamespaceIdentifier(id, currentDeclaration()->range().start) );
     }
     closeDeclaration();
   }
@@ -1002,7 +1129,8 @@ void DeclarationBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST
   bool openedDeclaration = false;
 
   if (node->name) {
-    QualifiedIdentifier id = identifierForNode(node->name);
+    QualifiedIdentifier id;
+    identifierForNode(node->name, id);
 
     bool forwardDeclarationGlobal = false;
 
@@ -1202,6 +1330,7 @@ void DeclarationBuilder::parseFunctionSpecifiers(const ListNode<std::size_t>* fu
 
 void DeclarationBuilder::visitParameterDeclaration(ParameterDeclarationAST* node) {
   DeclarationBuilderBase::visitParameterDeclaration(node);
+  
   AbstractFunctionDeclaration* function = currentDeclaration<AbstractFunctionDeclaration>();
 
   if( function ) {
@@ -1284,10 +1413,15 @@ bool DeclarationBuilder::checkParameterDeclarationClause(ParameterDeclarationCla
       if(ast) {
         if(m_collectQtFunctionSignature) {
           size_t endToken = ast->end_token;
-          if(ast->expression)
-            endToken = ast->expression->start_token;
-          if(ast->declarator && ast->declarator->id)
-            endToken = ast->declarator->id->start_token;
+          
+          if(ast->type_specifier)
+            endToken = ast->type_specifier->end_token;
+          if(ast->declarator) {
+            if(ast->declarator->id)
+              endToken = ast->declarator->id->start_token;
+            else
+              endToken = ast->declarator->end_token;
+          }
           
           if(!m_qtFunctionSignature.isEmpty())
             m_qtFunctionSignature += ", ";
