@@ -347,7 +347,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
           isConst = isConstant(pnt.cast<AbstractType>());
           //It is a pointer, reduce the pointer-depth by one
           m_lastType = pnt->baseType();
-          m_lastInstance = Instance( getDeclaration(node, m_lastType) );
+          m_lastInstance = Instance( getDeclaration(m_lastType) );
         } else {
           findMember( node, m_lastType, Identifier("operator->") ); ///@todo respect const
           if( !m_lastType ) {
@@ -405,7 +405,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
       if( constant )
         (*constant) |= (pnt->modifiers() & AbstractType::ConstModifier);
       m_lastType = pnt->baseType();
-      m_lastInstance = Instance(getDeclaration(node, m_lastType));
+      m_lastInstance = Instance(getDeclaration(m_lastType));
       return true;
     } else {
       LOCKDUCHAIN;
@@ -414,8 +414,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
     }
   }
 
-  Declaration* ExpressionVisitor::getDeclaration( AST* node, const AbstractType::Ptr& base ) {
-    Q_UNUSED(node)
+  Declaration* ExpressionVisitor::getDeclaration( const AbstractType::Ptr& base ) {
     if( !base ) return 0;
 
     const IdentifiedType* idType = dynamic_cast<const IdentifiedType*>(base.unsafeData());
@@ -530,7 +529,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
     PushPositiveContext pushContext( m_currentContext, node->ducontext );
 
     clearLast();
-
+    
     if( node->literal ) {
       visit( node->literal );
       return; //We had a string-literal
@@ -806,7 +805,13 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   for( size_t s = node->start_token; s < node->end_token; ++s )
     id += m_session->token_stream->token(s).symbolString();
 
-  QualifiedIdentifier ident( id );
+  //We have  to prevent automatic parsing and splitting by QualifiedIdentifier and Identifier
+  Identifier idd;
+  idd.setIdentifier(id);
+  
+  QualifiedIdentifier ident;
+  ident.push(idd);
+  
   ident.setIsExpression( expression );
   type->setIdentifier( ident );
   m_lastType = type.cast<AbstractType>();
@@ -902,7 +907,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       m_lastType = leftType;
       return;
     }
-
+    
     if( dynamic_cast<DelayedType*>(rightType.unsafeData()) || dynamic_cast<DelayedType*>(leftType.unsafeData()) ) {
       m_lastInstance = Instance(true);
       createDelayedType(node);
@@ -910,6 +915,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     }
 
     int tokenKind = tokenFromIndex(node->op).kind;
+    
     if(rightType && leftType && rightInstance && leftInstance) {
       LOCKDUCHAIN;
       //Test if there is a builtin operator that can be used. If it is, this will also evaluate the values of constant expressions.
@@ -972,7 +978,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   {
     PushPositiveContext pushContext( m_currentContext, ast->ducontext );
 
-    ///@todo cv-qualifiers
     clearLast();
 
     TypeASTVisitor comp(m_session, this, m_currentContext, topContext());
@@ -982,10 +987,12 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     QList<DeclarationPointer> decls = comp.declarations();
 
+    m_lastType = comp.type();
+    
     if( !decls.isEmpty() )
     {
       m_lastDeclarations = decls;
-      m_lastType = decls.first()->abstractType();
+//       m_lastType = decls.first()->abstractType(); If we do this, we may lose modifiers and such
 
       if( decls.first()->kind() == Declaration::Type )
         m_lastInstance = Instance(false);
@@ -1011,7 +1018,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   void ExpressionVisitor::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST* node)
   {
     PushPositiveContext pushContext( m_currentContext, node->ducontext );
-
+///@todo Use TypeASTVisitor
     clearLast();
 
     if (node->integrals) {
@@ -1093,7 +1100,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     LOCKDUCHAIN;
     if( node->array_dimensions ) {
-      ///@todo cv-qualifiers
       ArrayType::Ptr p( new ArrayType() );
       p->setElementType( m_lastType );
 
@@ -1168,11 +1174,27 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     }
 
     LOCKDUCHAIN;
-    ///@todo cv-qualifiers
-    PointerType::Ptr p( new PointerType() );
-    p->setBaseType( m_lastType );
-
-    m_lastType = p.cast<AbstractType>();
+    
+    static IndexedString ref("&");
+    static IndexedString ptr("*");
+    
+    IndexedString op = m_session->token_stream->token(node->op).symbol();
+    
+    
+    if(op == ptr) {
+    
+      PointerType::Ptr p( new PointerType() );
+      p->setBaseType( m_lastType );
+      p->setModifiers(TypeBuilder::parseConstVolatile(m_session, node->cv));
+  
+      m_lastType = p.cast<AbstractType>();
+    }else{
+      ReferenceType::Ptr p( new ReferenceType() );
+      p->setBaseType( m_lastType );
+      p->setModifiers(TypeBuilder::parseConstVolatile(m_session, node->cv));
+  
+      m_lastType = p.cast<AbstractType>();
+    }
     m_lastInstance = Instance(false);
   }
 
@@ -1256,8 +1278,10 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     //Also visit the not interesting parts, so they are evaluated
     clearLast();
+    
     visit(node->condition);
 
+    
     if( dynamic_cast<DelayedType*>(m_lastType.unsafeData()) ) {
       //Store the expression so it's evaluated later
       m_lastInstance = Instance(true);
@@ -1330,20 +1354,20 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
   bool ExpressionVisitor::dereferenceLastPointer(AST* node) {
     if( PointerType::Ptr pt = realLastType().cast<PointerType>() )
-    { ///@todo what about const in pointer?
+    {
       //Dereference
       m_lastType = pt->baseType();
-      m_lastInstance = Instance( getDeclaration(node,m_lastType) );
+      m_lastInstance = Instance( getDeclaration(m_lastType) );
       return true;
     }else if( ArrayType::Ptr pt = realLastType().cast<ArrayType>() ) {
       m_lastType = pt->elementType();
-      m_lastInstance = Instance( getDeclaration(node,m_lastType) );
+      m_lastInstance = Instance( getDeclaration(m_lastType) );
       return true;
     }else{
       return false;
     }
   }
-
+  
   /**
    * partially have test */
   void ExpressionVisitor::visitUnaryExpression(UnaryExpressionAST* node)
@@ -1392,13 +1416,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     break;
     case '&':
     {
-      bool constant = false;
-      AbstractType::Ptr oldType = realLastType(&constant); ///Dereference references
-      PointerType::Ptr newPointer(new PointerType());
-      if (constant)
-        newPointer->setModifiers(AbstractType::ConstModifier);
-      newPointer->setBaseType( oldType );
-      m_lastType = newPointer.cast<AbstractType>();
+      m_lastType = increasePointerDepth(m_lastType);
       //m_lastInstance will be left alone as it was before. A pointer is not identified, and has no declaration.
     }
     break;
