@@ -108,7 +108,9 @@ void TypeBuilder::visitBaseSpecifier(BaseSpecifierAST *node)
     if( openedType ) {
       closeType();
     } else { //A case for the problem-reporter
-      kDebug(9007) << "Could not find base declaration for" << identifierForNode(node->name);
+      QualifiedIdentifier id;
+      identifierForNode(node->name, id);
+      kDebug(9007) << "Could not find base declaration for" << id;
     }
   }
 
@@ -136,7 +138,6 @@ void TypeBuilder::visitEnumerator(EnumeratorAST* node)
     Cpp::ExpressionEvaluationResult res;
 
     bool delay = false;
-
     if(!delay) {
       DUChainReadLocker lock(DUChain::lock());
       node->expression->ducontext = currentContext();
@@ -145,14 +146,19 @@ void TypeBuilder::visitEnumerator(EnumeratorAST* node)
       //Delay the type-resolution of template-parameters
       if( res.allDeclarations.size() ) {
         Declaration* decl = res.allDeclarations[0].getDeclaration(currentContext()->topContext());
-        if( dynamic_cast<TemplateParameterDeclaration*>(decl) || isTemplateDependent(decl))
+        if( dynamic_cast<TemplateParameterDeclaration*>(decl) || isTemplateDependent(decl)) {
           delay = true;
+        }
       }
 
       if ( !delay && res.isValid() && res.isInstance ) {
         AbstractType::Ptr resType = res.type.type();
         if( ConstantIntegralType::Ptr iType = resType.cast<ConstantIntegralType>() ) {
           m_currentEnumeratorValue = (int)iType->value<qint64>();
+          EnumeratorType::Ptr enumerator(new EnumeratorType());
+          enumerator->setValue<qint64>(m_currentEnumeratorValue);
+          openedType = true;
+          openType(enumerator);
         } else if( DelayedType::Ptr dType = resType.cast<DelayedType>() ) {
           openType(dType.cast<AbstractType>()); ///@todo Make this an enumerator-type that holds the same information
           openedType = true;
@@ -206,7 +212,7 @@ void TypeBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST *node)
 
   if( kind == Token_typename ) {
     //For typename, just find the type and return
-    bool openedType = openTypeFromName(node->name, parseConstVolatile(node->cv));
+    bool openedType = openTypeFromName(node->name, parseConstVolatile(editor()->parseSession(), node->cv));
 
     TypeBuilderBase::visitElaboratedTypeSpecifier(node);
 
@@ -315,7 +321,7 @@ void TypeBuilder::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST *node)
     if(type == IntegralType::TypeNone)
       type = IntegralType::TypeInt; //Happens, example: "unsigned short"
 
-    modifiers |= parseConstVolatile(node->cv);
+    modifiers |= parseConstVolatile(editor()->parseSession(), node->cv);
 
     IntegralType::Ptr integral(new IntegralType(type));
     integral->setModifiers(modifiers);
@@ -323,7 +329,7 @@ void TypeBuilder::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST *node)
     openType(integral);
     
   } else if (node->name) {
-    openedType = openTypeFromName(node->name, parseConstVolatile(node->cv));
+    openedType = openTypeFromName(node->name, parseConstVolatile(editor()->parseSession(), node->cv));
   }
 
   TypeBuilderBase::visitSimpleTypeSpecifier(node);
@@ -381,7 +387,8 @@ void TypeBuilder::closeTypeForInitializer(InitializerAST */*node*/) {
 }
 
 bool TypeBuilder::openTypeFromName(NameAST* name, uint modifiers, bool needClass) {
-  QualifiedIdentifier id = identifierForNode(name);
+  QualifiedIdentifier id;
+  identifierForNode(name, id);
 
   bool openedType = false;
 
@@ -509,14 +516,14 @@ void TypeBuilder::visitPtrOperator(PtrOperatorAST* node)
     if (!op.isEmpty())
       if (op[0] == '&') {
         ReferenceType::Ptr pointer(new ReferenceType());
-        pointer->setModifiers(parseConstVolatile(node->cv));
+        pointer->setModifiers(parseConstVolatile(editor()->parseSession(), node->cv));
         pointer->setBaseType(lastType());
         openType(pointer);
         typeOpened = true;
 
       } else if (op[0] == '*') {
         PointerType::Ptr pointer(new PointerType());
-        pointer->setModifiers(parseConstVolatile(node->cv));
+        pointer->setModifiers(parseConstVolatile(editor()->parseSession(), node->cv));
         pointer->setBaseType(lastType());
         openType(pointer);
         typeOpened = true;
@@ -534,7 +541,7 @@ FunctionType* TypeBuilder::openFunction(DeclaratorAST *node)
   FunctionType* functionType = new FunctionType();
 
   if (node->fun_cv)
-    functionType->setModifiers(parseConstVolatile(node->fun_cv));
+    functionType->setModifiers(parseConstVolatile(editor()->parseSession(), node->fun_cv));
 
   if (lastType())
     functionType->setReturnType(lastType());
@@ -597,7 +604,7 @@ void TypeBuilder::visitArrayExpression(ExpressionAST* expression)
     closeType();
 }
 
-uint TypeBuilder::parseConstVolatile(const ListNode<std::size_t> *cv)
+uint TypeBuilder::parseConstVolatile(ParseSession* session, const ListNode<std::size_t> *cv)
 {
   uint ret = AbstractType::NoModifiers;
 
@@ -605,7 +612,7 @@ uint TypeBuilder::parseConstVolatile(const ListNode<std::size_t> *cv)
     const ListNode<std::size_t> *it = cv->toFront();
     const ListNode<std::size_t> *end = it;
     do {
-      int kind = editor()->parseSession()->token_stream->kind(it->element);
+      int kind = session->token_stream->kind(it->element);
       if (kind == Token_const)
         ret |= AbstractType::ConstModifier;
       else if (kind == Token_volatile)
@@ -626,14 +633,15 @@ void TypeBuilder::openDelayedType(const TypeIdentifier& identifier, AST* /*node*
   openType(type);
 }
 
-
 void TypeBuilder::visitTemplateParameter(TemplateParameterAST *ast)
 {
-  openType(CppTemplateParameterType::Ptr(new CppTemplateParameterType()));
+//   if(!ast->parameter_declaration)
+    openType(CppTemplateParameterType::Ptr(new CppTemplateParameterType()));
 
   TypeBuilderBase::visitTemplateParameter(ast);
-
-  closeType();
+  
+//   if(!ast->parameter_declaration)
+    closeType();
 }
 
 
