@@ -47,6 +47,8 @@ pp::pp(Preprocessor* preprocessor)
   , nextToken(0)
   , haveNextToken(false)
   , hideNext(false)
+  , hadGuardCandidate(false)
+  , checkGuardEnd(false)
 {
   iflevel = 0;
   _M_skipping[iflevel] = 0;
@@ -111,15 +113,22 @@ uint includeNextDirective = KDevelop::IndexedString("include_next").index();
 void pp::handle_directive(uint directive, Stream& input, Stream& output)
 {
   skip_blanks (input, output);
-
+  if(!(directive == ifndefDirective)) {
+    hadGuardCandidate = true; //Too late, the guard must be the first directive
+  }
+  if(checkGuardEnd) {
+    guardCandidate = KDevelop::IndexedString();
+    checkGuardEnd = false;
+  }
+  
   if(directive == defineDirective)
       if (! skipping ())
         return handle_define(input);
-  
+
   if(directive == includeDirective || directive == includeNextDirective)
       if (! skipping ())
         return handle_include (directive == includeNextDirective, input, output);
-  
+
   if(directive == undefDirective)
       if (! skipping ())
         return handle_undef(input);
@@ -224,7 +233,7 @@ void pp::operator () (Stream& input, Stream& output)
 
       Anchor inputPosition = input.inputPosition();
       KDevelop::SimpleCursor originalInputPosition = input.originalInputPosition();
-      
+
       PreprocessedContents skipped;
       {
         Stream ss(&skipped);
@@ -244,18 +253,31 @@ void pp::operator () (Stream& input, Stream& output)
 
     } else {
       output.mark(input.inputPosition());
+      if(checkGuardEnd) {
+        expand.startSignificantContentSearch();
+      }
+      
       expand (input, output);
+      if(checkGuardEnd) {
+        if(expand.foundSignificantContent() || !input.atEnd()) {
+          guardCandidate = KDevelop::IndexedString();
+        }
+        checkGuardEnd = false;
+      }
     }
   }
+  
+  if(!guardCandidate.isEmpty())
+    preprocessor()->foundHeaderGuard(input, guardCandidate);
 
   if (iflevel != previousIfLevel && !input.skippedToEnd())
     createProblem(input, i18n("Unterminated #if statement"));
 }
 
 void pp::createProblem(Stream& input, const QString& description) {
-    KDevelop::Problem problem;
-    problem.setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
-    problem.setDescription(description);
+    KDevelop::ProblemPointer problem(new KDevelop::Problem);
+    problem->setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
+    problem->setDescription(description);
     problemEncountered(problem);
 }
 
@@ -319,7 +341,7 @@ void pp::handle_define (Stream& input)
   {
     if(input == '/' && (input.peekNextCharacter() == '/' || input.peekNextCharacter() == '*')) {
       skip_comment_or_divop(input, devnull());
-      if(input != '\n')
+      if(!input.atEnd() && input != '\n')
         skip_blanks (input, devnull());
       continue;
     }
@@ -426,11 +448,11 @@ Value pp::eval_primary(Stream& input)
 
       if (token != TOKEN_IDENTIFIER)
       {
-        KDevelop::Problem problem;
-        problem.setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 1)));
+        KDevelop::ProblemPointer problem(new KDevelop::Problem);
+        problem->setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 1)));
         QChar tk(token);
-        problem.setDescription(i18n("Expected \"identifier\", found: %1", (tk < TOKENS_END && tk > TOKENS_START) ? QString(tk) : i18n("character %1", token)));
-        problem.setExplanation(i18n("<h5>Token text</h5><pre>%1</pre><h5>Input</h5><pre>%2</pre>", token_text.str(), QString::fromUtf8(input.stringFrom(start))));
+        problem->setDescription(i18n("Expected \"identifier\", found: %1", (tk < TOKENS_END && tk > TOKENS_START) ? QString(tk) : i18n("character %1", token)));
+        problem->setExplanation(i18n("<h5>Token text</h5><pre>%1</pre><h5>Input</h5><pre>%2</pre>", token_text.str(), QString::fromUtf8(input.stringFrom(start))));
         problemEncountered(problem);
         break;
       }
@@ -444,11 +466,11 @@ Value pp::eval_primary(Stream& input)
 
       if (expect_paren) {
         if (token != ')') {
-          KDevelop::Problem problem;
-          problem.setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
+          KDevelop::ProblemPointer problem(new KDevelop::Problem);
+          problem->setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
           QChar tk(token);
-          problem.setDescription(i18n("Expected \")\", found %1", tk.isLetterOrNumber() ? QString(tk) : i18n("character %1", token)));
-          problem.setExplanation(i18n("<h5>Token text</h5><pre>%1</pre><h5>Input</h5><pre>%2</pre>", token_text.str(), QString::fromUtf8(input.stringFrom(start))));
+          problem->setDescription(i18n("Expected \")\", found %1", tk.isLetterOrNumber() ? QString(tk) : i18n("character %1", token)));
+          problem->setExplanation(i18n("<h5>Token text</h5><pre>%1</pre><h5>Input</h5><pre>%2</pre>", token_text.str(), QString::fromUtf8(input.stringFrom(start))));
           problemEncountered(problem);
         } else {
           accept_token();
@@ -476,11 +498,11 @@ Value pp::eval_primary(Stream& input)
       token = next_token(input);
 
       if (token != ')') {
-        KDevelop::Problem problem;
-        problem.setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 1)));
+        KDevelop::ProblemPointer problem(new KDevelop::Problem);
+        problem->setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 1)));
         QChar tk(token);
-        problem.setDescription(i18n("Expected \")\", found %1", tk.isLetterOrNumber() ? QString(tk) : i18n("character %1", token)));
-        problem.setExplanation(i18n("<h5>Token text</h5><pre>%1</pre><h5>Input</h5><pre>%2</pre>", token_text.str(), QString::fromUtf8(input.stringFrom(start))));
+        problem->setDescription(i18n("Expected \")\", found %1", tk.isLetterOrNumber() ? QString(tk) : i18n("character %1", token)));
+        problem->setExplanation(i18n("<h5>Token text</h5><pre>%1</pre><h5>Input</h5><pre>%2</pre>", token_text.str(), QString::fromUtf8(input.stringFrom(start))));
         problemEncountered(problem);
       } else {
         accept_token();
@@ -513,10 +535,10 @@ Value pp::eval_multiplicative(Stream& input)
 
     } else if (token == '/') {
       if (value.is_zero()) {
-        KDevelop::Problem problem;
-        problem.setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
-        problem.setDescription(i18n("Division by zero"));
-        problem.setDescription(i18n("Input text: %1", QString::fromUtf8(input.stringFrom(start))));
+        KDevelop::ProblemPointer problem(new KDevelop::Problem);
+        problem->setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
+        problem->setDescription(i18n("Division by zero"));
+        problem->setDescription(i18n("Input text: %1", QString::fromUtf8(input.stringFrom(start))));
         problemEncountered(problem);
         result.set_long(0);
 
@@ -526,10 +548,10 @@ Value pp::eval_multiplicative(Stream& input)
 
     } else {
       if (value.is_zero()) {
-        KDevelop::Problem problem;
-        problem.setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
-        problem.setDescription(i18n("Division by zero"));
-        problem.setDescription(i18n("Input text: %1", QString::fromUtf8(input.stringFrom(start))));
+        KDevelop::ProblemPointer problem(new KDevelop::Problem);
+        problem->setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
+        problem->setDescription(i18n("Division by zero"));
+        problem->setDescription(i18n("Input text: %1", QString::fromUtf8(input.stringFrom(start))));
         problemEncountered(problem);
         result.set_long(0);
 
@@ -764,9 +786,9 @@ Value pp::eval_constant_expression(Stream& input)
     }
     else
     {
-      KDevelop::Problem problem;
-      problem.setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 1)));
-      problem.setDescription(i18n("expected ``:'' = %1", int(token)));
+      KDevelop::ProblemPointer problem(new KDevelop::Problem);
+      problem->setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 1)));
+      problem->setDescription(i18n("expected ``:'' = %1", int(token)));
       problemEncountered(problem);
       result = left_value;
     }
@@ -827,11 +849,14 @@ void pp::handle_if (Stream& input)
 
 void pp::handle_else(int sourceLine)
 {
+  if(iflevel == 1)
+    guardCandidate = KDevelop::IndexedString();
+
   if (iflevel == 0 && !skipping ())
   {
-    KDevelop::Problem problem;
-    problem.setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(KTextEditor::Cursor(sourceLine, 0), 0)));
-    problem.setDescription(i18n("#else without #if"));
+    KDevelop::ProblemPointer problem(new KDevelop::Problem);
+    problem->setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(KTextEditor::Cursor(sourceLine, 0), 0)));
+    problem->setDescription(i18n("#else without #if"));
     problemEncountered(problem);
   }
   else if (iflevel > 0 && _M_skipping[iflevel - 1])
@@ -849,13 +874,16 @@ void pp::handle_else(int sourceLine)
 
 void pp::handle_elif(Stream& input)
 {
+  if(iflevel == 1)
+    guardCandidate = KDevelop::IndexedString();
+  
   RETURN_ON_FAIL(iflevel > 0);
 
   if (iflevel == 0 && !skipping())
   {
-    KDevelop::Problem problem;
-    problem.setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
-    problem.setDescription(i18n("#else without #if"));
+    KDevelop::ProblemPointer problem(new KDevelop::Problem);
+    problem->setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
+    problem->setDescription(i18n("#else without #if"));
     problemEncountered(problem);
   }
   else
@@ -893,9 +921,9 @@ void pp::handle_endif(Stream& input, Stream& output)
 {
   if (iflevel == 0 && !skipping())
   {
-    KDevelop::Problem problem;
-    problem.setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
-    problem.setDescription(i18n("#endif without #if at output line %1", m_environment->locationTable()->anchorForOffset(output.offset()).anchor.line));
+    KDevelop::ProblemPointer problem(new KDevelop::Problem);
+    problem->setFinalLocation(KDevelop::DocumentRange(currentFileNameString(), KTextEditor::Range(input.originalInputPosition().textCursor(), 0)));
+    problem->setDescription(i18n("#endif without #if at output line %1", m_environment->locationTable()->anchorForOffset(output.offset()).anchor.line));
     problemEncountered(problem);
   }
   else
@@ -906,6 +934,11 @@ void pp::handle_endif(Stream& input, Stream& output)
     _M_true_test[iflevel] = 0;
 
     --iflevel;
+    if(iflevel == 0) {
+      if(!guardCandidate.isEmpty()) {
+        checkGuardEnd = true;
+      }
+    }
   }
 }
 
@@ -926,6 +959,13 @@ void pp::handle_ifdef (bool check_undefined, Stream& input)
 {
   KDevelop::IndexedString macro_name(skip_identifier(input));
 ///@todo eventually fix the block description
+  if(check_undefined && expand.in_header_section() && guardCandidate.isEmpty() && !hadGuardCandidate && iflevel == 0) {
+    //It's the first #ifndef and the header-section hasn't ended yet, assume it to be the header-guard
+    guardCandidate = macro_name;
+  }
+
+  hadGuardCandidate = true;
+  
   environment()->enterBlock(input.inputPosition().line);//, QString("%1defined(%2)").arg(check_undefined ? "!" : "").arg(macro_name).toUtf8());
 
   if (test_if_level())
@@ -959,7 +999,7 @@ void pp::handle_undef(Stream& input)
   macro.defined = false;
 
   m_environment->setMacro(makeConstant(&macro));
-  
+
   //m_environment->clearMacro(macro_name);
 }
 
@@ -1160,11 +1200,6 @@ const QList< KDevelop::ProblemPointer > & rpp::pp::problems() const
   return m_problems;
 }
 
-void rpp::pp::problemEncountered(const KDevelop::Problem & problem)
-{
-  problemEncountered(KDevelop::ProblemPointer(new KDevelop::Problem(problem)));
-}
-
 void rpp::pp::problemEncountered(const KDevelop::ProblemPointer & problem)
 {
   m_problems.append(problem);
