@@ -178,14 +178,27 @@ void TestCppCodeCompletion::testPrivateVariableCompletion() {
   
   QList <Item > items = cptr->completionItems(context->range().end, abort);
   QStandardItemModel fakeModel;
+  fakeModel.setColumnCount(10);
+  fakeModel.setRowCount(10);
+  
+  bool hadThis = false;
+  
   foreach(Item i, items) {
     NormalDeclarationCompletionItem* decItem  = dynamic_cast<NormalDeclarationCompletionItem*>(i.data());
-    QVERIFY(decItem);
-    kDebug() << decItem->declaration()->toString();
     kDebug() << i->data(fakeModel.index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString();
+    if(decItem) {
+      kDebug() << typeid(*i.data()).name();
+      QVERIFY(decItem);
+      kDebug() << decItem->declaration()->toString();
+    }else{
+      TypeConversionCompletionItem* conversion = dynamic_cast<TypeConversionCompletionItem*>(i.data());
+      QVERIFY(conversion);
+      QCOMPARE(conversion->data(fakeModel.index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString(), QString("this"));
+      hadThis = true;
+    }
   }
-  
-  QCOMPARE(items.count(), 3); //C, test, and i
+  QVERIFY(hadThis);
+  QCOMPARE(items.count(), 4); //C, test, i, and "this"
 
   lock.lock();
   release(context);
@@ -721,6 +734,23 @@ void TestCppCodeCompletion::testAcrossHeaderReferences()
   release(top);
 }
 
+void TestCppCodeCompletion::testAcrossHeaderTemplateResolution() {
+  addInclude("acrossheaderresolution1.h", "class C {}; namespace std { template<class T> class A {  }; }");
+  addInclude("acrossheaderresolution2.h", "namespace std { template<class T> class B { typedef A<T> Type; }; }");
+  
+  QByteArray method("#include \"acrossheaderresolution1.h\"\n#include \"acrossheaderresolution2.h\"\n std::B<C>::Type t;");
+  
+  DUContext* top = parse(method, DumpNone);
+
+  DUChainWriteLocker lock(DUChain::lock());
+  
+  Declaration* decl = findDeclaration(top, QualifiedIdentifier("t"), top->range().end);
+  QVERIFY(decl);
+  QCOMPARE(QualifiedIdentifier(decl->abstractType()->toString()), QualifiedIdentifier("std::A<C>"));
+  
+  release(top);
+}
+
 void TestCppCodeCompletion::testAcrossHeaderTemplateReferences()
 {
   addInclude( "acrossheader1.h", "class Dummy { }; template<class Q> class Test{ };" );
@@ -863,8 +893,75 @@ void TestCppCodeCompletion::testMacroExpansionRanges() {
 }
 }
 
+void TestCppCodeCompletion::testNaiveMatching() {
+  return;
+    Cpp::EnvironmentManager::setMatchingLevel(Cpp::EnvironmentManager::Naive);
+    {
+      addInclude("recursive_test_1.h", "#include \"recursive_test_2.h\"\nint i1;\n");
+      addInclude("recursive_test_2.h", "#include \"recursive_test_1.h\"\nint i2;\n");
+      
+      TopDUContext* test1 = parse(QByteArray("#include \"recursive_test_1.h\"\n"), DumpNone);
+      DUChainWriteLocker l(DUChain::lock());
+      QCOMPARE(test1->recursiveImportIndices().count(), 3u);
+      QCOMPARE(test1->importedParentContexts().count(), 1);
+      QCOMPARE(test1->importedParentContexts()[0].indexedContext().context()->importedParentContexts().count(), 1);
+      QCOMPARE(test1->importedParentContexts()[0].indexedContext().context()->importedParentContexts()[0].indexedContext().context()->importedParentContexts().count(), 1);
+      QCOMPARE(test1->importedParentContexts()[0].indexedContext().context()->importedParentContexts()[0].indexedContext().context()->importedParentContexts()[0].indexedContext().context()->importedParentContexts().count(), 1);
+      Cpp::EnvironmentFile* envFile1 = dynamic_cast<Cpp::EnvironmentFile*>(test1->parsingEnvironmentFile().data());
+      QVERIFY(envFile1);
+      QVERIFY(envFile1->headerGuard().isEmpty());
+      release(test1);
+    }
+
+}
+
+void TestCppCodeCompletion::testHeaderGuards() {
+    {
+      TopDUContext* test1 = parse(QByteArray("#ifndef GUARD\n#define GUARD\nint x = 5; \n#endif\n#define BLA\n"), DumpNone);
+      DUChainWriteLocker l(DUChain::lock());
+      Cpp::EnvironmentFile* envFile1 = dynamic_cast<Cpp::EnvironmentFile*>(test1->parsingEnvironmentFile().data());
+      QVERIFY(envFile1);
+      QVERIFY(envFile1->headerGuard().isEmpty());
+      release(test1);
+    }
+    {
+      TopDUContext* test1 = parse(QByteArray("#ifndef GUARD\n#define GUARD\nint x = 5;\n#ifndef GUARD\n#define GUARD\n#endif\n#if defined(TEST)\n int q = 4;#endif\n#endif\n"), DumpNone);
+      DUChainWriteLocker l(DUChain::lock());
+      Cpp::EnvironmentFile* envFile1 = dynamic_cast<Cpp::EnvironmentFile*>(test1->parsingEnvironmentFile().data());
+      QVERIFY(envFile1);
+      QCOMPARE(envFile1->headerGuard().str(), QString("GUARD"));
+      release(test1);
+    }
+    {
+      TopDUContext* test1 = parse(QByteArray("int x;\n#ifndef GUARD\n#define GUARD\nint x = 5; \n#endif\n"), DumpNone);
+      DUChainWriteLocker l(DUChain::lock());
+      Cpp::EnvironmentFile* envFile1 = dynamic_cast<Cpp::EnvironmentFile*>(test1->parsingEnvironmentFile().data());
+      QVERIFY(envFile1);
+      QVERIFY(envFile1->headerGuard().isEmpty());
+      release(test1);
+    }
+    {
+      TopDUContext* test1 = parse(QByteArray("#define X\n#ifndef GUARD\n#define GUARD\nint x = 5; \n#endif\n"), DumpNone);
+      DUChainWriteLocker l(DUChain::lock());
+      Cpp::EnvironmentFile* envFile1 = dynamic_cast<Cpp::EnvironmentFile*>(test1->parsingEnvironmentFile().data());
+      QVERIFY(envFile1);
+      QVERIFY(envFile1->headerGuard().isEmpty());
+      release(test1);
+    }
+    {
+      TopDUContext* test1 = parse(QByteArray("#ifndef GUARD\n#define GUARD\nint x = 5; \n#endif\nint o;\n"), DumpNone);
+      DUChainWriteLocker l(DUChain::lock());
+      Cpp::EnvironmentFile* envFile1 = dynamic_cast<Cpp::EnvironmentFile*>(test1->parsingEnvironmentFile().data());
+      QVERIFY(envFile1);
+      QVERIFY(envFile1->headerGuard().isEmpty());
+      release(test1);
+    }
+}
+
 void TestCppCodeCompletion::testEnvironmentMatching() {
     {
+      CppPreprocessEnvironment::setRecordOnlyImportantString(false);
+      
       addInclude("deep2.h", "#ifdef WANT_DEEP\nint x;\n#undef WANT_DEEP\n#endif\n");
       addInclude("deep1.h", "#define WANT_DEEP\n#include \"deep2.h\"\n");
       TopDUContext* test1 = parse(QByteArray("#include \"deep1.h\""), DumpNone);
@@ -1101,6 +1198,10 @@ struct TestPreprocessor : public rpp::Preprocessor {
     if(stopAfterHeaders)
       stream.toEnd();
   }
+  
+  virtual void foundHeaderGuard(rpp::Stream& stream, KDevelop::IndexedString guardName) {
+    environmentFile->setHeaderGuard(guardName);
+  }
 };
 
 QString TestCppCodeCompletion::preprocess( const HashedString& url, const QString& text, IncludeFileList& included, rpp::pp* parent, bool stopAfterHeaders, KSharedPtr<Cpp::EnvironmentFile>* paramEnvironmentFile, rpp::LocationTable** returnLocationTable, PreprocessedContents* targetContents ) {
@@ -1141,7 +1242,7 @@ QString TestCppCodeCompletion::preprocess( const HashedString& url, const QStrin
     if (returnLocationTable)
       *returnLocationTable = preprocessor.environment()->takeLocationTable();
 
-    currentEnvironment->finish();
+    currentEnvironment->finishEnvironment();
 
     if( parent ) {
       preprocessor.environment()->swapMacros(parent->environment());
