@@ -70,12 +70,15 @@
 #include "cmakecodecompletionmodel.h"
 #include "icmakebuilder.h"
 
-#include "ui_cmakepossibleroots.h"
-
 #ifdef CMAKEDEBUGVISITOR
 #include "cmakedebugvisitor.h"
 #endif
 
+#include "ui_cmakepossibleroots.h"
+#include <language/editor/editorintegrator.h>
+#include <language/duchain/smartconverter.h>
+#include <language/duchain/use.h>
+
 using namespace KDevelop;
 
 K_PLUGIN_FACTORY(CMakeSupportFactory, registerPlugin<CMakeProjectManager>(); )
@@ -364,8 +367,6 @@ KDevelop::ProjectFolderItem* CMakeProjectManager::import( KDevelop::IProject *pr
         m_projectCache[project]=readCache(cachefile);
         initializeProject(project, folderUrl);
 
-        m_folderPerUrl[folderUrl]=m_rootItem;
-        
         m_watchers[project]->disconnect( SIGNAL(dirty() ), this, SLOT(dirtyFile(QString)));
         connect(m_watchers[project], SIGNAL(dirty(QString)), this, SLOT(dirtyFile(QString)));
         Q_ASSERT(m_rootItem->rowCount()==0);
@@ -427,7 +428,6 @@ QList<KDevelop::ProjectFolderItem*> CMakeProjectManager::parse( KDevelop::Projec
     if(folder && folder->type()==KDevelop::ProjectBaseItem::BuildFolder)
     {
         Q_ASSERT(folder->rowCount()==0);
-        m_folderPerUrl[item->url()]=folder;
 
         kDebug(9042) << "parse:" << folder->url();
         KUrl cmakeListsPath(folder->url());
@@ -441,161 +441,175 @@ QList<KDevelop::ProjectFolderItem*> CMakeProjectManager::parse( KDevelop::Projec
         if(f.isEmpty())
         {
             kDebug() << "There is no" << cmakeListsPath;
-            return folderList;
-        }
-        kDebug(9042) << "Adding cmake: " << cmakeListsPath << " to the model";
-
-        m_watchers[item->project()]->addFile(cmakeListsPath.toLocalFile());
-        QString binDir=KUrl::relativePath(m_realRoot[folder->project()].toLocalFile(), folder->url().toLocalFile());
-        if(binDir.startsWith("./"))
-            binDir=binDir.remove(0, 2);
-        QString currentBinDir=vm->value("CMAKE_BINARY_DIR")[0]+binDir;
-        
-        vm->insert("CMAKE_CURRENT_BINARY_DIR", QStringList(currentBinDir));
-        vm->insert("CMAKE_CURRENT_LIST_FILE", QStringList(cmakeListsPath.toLocalFile(KUrl::RemoveTrailingSlash)));
-        vm->insert("CMAKE_CURRENT_SOURCE_DIR", QStringList(folder->url().toLocalFile(KUrl::RemoveTrailingSlash)));
-
-        kDebug(9042) << "currentBinDir" << KUrl(vm->value("CMAKE_BINARY_DIR")[0]) << vm->value("CMAKE_CURRENT_BINARY_DIR");
-        
-    #ifdef CMAKEDEBUGVISITOR
-        CMakeAstDebugVisitor dv;
-        dv.walk(cmakeListsPath.toLocalFile(), f, 0);
-    #endif
-
-        ReferencedTopDUContext curr;
-        if(folder->parent())
-        {
-            curr=dynamic_cast<CMakeFolderItem*>(folder->parent())->topDUContext();
         }
         else
-            curr=m_buildstrapContext;
-        CMakeProjectVisitor v(folder->url().toLocalFile(KUrl::RemoveTrailingSlash), curr);
-        v.setCacheValues(&m_projectCache[item->project()]);
-        v.setVariableMap(vm);
-        v.setMacroMap(mm);
-        v.setModulePath(m_modulePathPerProject[item->project()]);
-        v.setDefinitions(folder->definitions());
-        v.walk(f, 0);
-        folder->setTopDUContext(v.context());
-        vm->remove("CMAKE_CURRENT_LIST_FILE");
-        vm->remove("CMAKE_CURRENT_SOURCE_DIR");
-        vm->remove("CMAKE_CURRENT_BINARY_DIR");
-
-        /*{
-        kDebug() << "dumpiiiiiing" << folder->url();
-        KDevelop::DUChainReadLocker lock(KDevelop::DUChain::lock());
-        KDevelop::DumpChain dump;
-        dump.dump( v.context(), false);
-        }*/
-
-        if(folder->text()=="/" && !v.projectName().isEmpty())
         {
-            folder->setText(v.projectName());
-        }
+            kDebug(9042) << "Adding cmake: " << cmakeListsPath << " to the model";
 
-        KUrl subroot=m_subprojectRoot[item->project()];
-        
-        foreach (const QString& subf, v.subdirectories())
-        {
-            if(subf.isEmpty()) //This would not be necessary if we didn't parse the wrong lines
-                continue;
-            KUrl path(folder->url());
-            path.addPath(subf);
+            m_watchers[item->project()]->addFile(cmakeListsPath.toLocalFile());
+            QString binDir=KUrl::relativePath(m_realRoot[folder->project()].toLocalFile(), folder->url().toLocalFile());
+            if(binDir.startsWith("./"))
+                binDir=binDir.remove(0, 2);
+            QString currentBinDir=vm->value("CMAKE_BINARY_DIR")[0]+binDir;
             
-            kDebug(9042) << "Found subdir " << path << "which should be into" << subroot;
-            if(subroot.isParentOf(path) || path.isParentOf(subroot))
+            vm->insert("CMAKE_CURRENT_BINARY_DIR", QStringList(currentBinDir));
+            vm->insert("CMAKE_CURRENT_LIST_FILE", QStringList(cmakeListsPath.toLocalFile(KUrl::RemoveTrailingSlash)));
+            vm->insert("CMAKE_CURRENT_SOURCE_DIR", QStringList(folder->url().toLocalFile(KUrl::RemoveTrailingSlash)));
+
+            kDebug(9042) << "currentBinDir" << KUrl(vm->value("CMAKE_BINARY_DIR")[0]) << vm->value("CMAKE_CURRENT_BINARY_DIR");
+            
+        #ifdef CMAKEDEBUGVISITOR
+            CMakeAstDebugVisitor dv;
+            dv.walk(cmakeListsPath.toLocalFile(), f, 0);
+        #endif
+
+            ReferencedTopDUContext curr;
+            if(folder->parent())
             {
-                if(entries.contains(subf))
-                {
-                    entries.removeAll(subf);
-                }
-                
-                CMakeFolderItem* a = new CMakeFolderItem( folder->project(), subf, folder );
-                kDebug() << "folder: " << a << a->index();
-                a->setUrl(path);
-                a->setDefinitions(v.definitions());
-                folderList.append( a );
-                
-                DescriptorAttatched* datt=static_cast<DescriptorAttatched*>(a);
-                datt->setDescriptor(v.folderDeclarationDescriptor(subf));
+                curr=dynamic_cast<CMakeFolderItem*>(folder->parent())->topDUContext();
             }
-        }
-        
-//         if(folderList.isEmpty() && path.isParentOf(item->url()))
-//             kDebug() << "poor guess";
-        
-        QStringList directories;
-        directories += folder->url().toLocalFile(KUrl::RemoveTrailingSlash);
-        directories += currentBinDir;
-
-        foreach(const QString& s, v.includeDirectories())
-        {
-            QString dir(s);
-            if(!s.startsWith('/') && !s.startsWith("#["))
-            {
-                KUrl path=folder->url();
-                path.addPath(s);
-                dir=path.toLocalFile();
+            else
+                curr=m_buildstrapContext;
+            CMakeProjectVisitor v(folder->url().toLocalFile(KUrl::RemoveTrailingSlash), curr);
+            v.setCacheValues(&m_projectCache[item->project()]);
+            v.setVariableMap(vm);
+            v.setMacroMap(mm);
+            v.setModulePath(m_modulePathPerProject[item->project()]);
+            v.setDefinitions(folder->definitions());
+            v.walk(f, 0);
+            folder->setTopDUContext(v.context());
+            vm->remove("CMAKE_CURRENT_LIST_FILE");
+            vm->remove("CMAKE_CURRENT_SOURCE_DIR");
+            vm->remove("CMAKE_CURRENT_BINARY_DIR");
+            
+            EditorIntegrator editor;
+            editor.setCurrentUrl(IndexedString(v.context()->url().toUrl()));
+            LockedSmartInterface smart(editor.smart());
+            if(smart) {
+                DUChainWriteLocker lock(DUChain::lock());
+                SmartConverter converter(&editor);
+                converter.convertDUChain(v.context());
+                CMakeHighlighting highlight;
+                highlight.highlightDUChain(v.context());
             }
-            
-            KUrl simp(dir); //We use this to simplify dir
-            simp.cleanPath();
-            dir=simp.toLocalFile();
-            
-            if(!directories.contains(dir))
-                directories.append(dir);
-        }
-        folder->setIncludeDirectories(directories);
-        folder->setDefinitions(v.definitions());
 
-        foreach ( const QString& t, v.targets())
-        {
-            QStringList dependencies=v.targetDependencies(t);
-            if(!dependencies.isEmpty()) //Just to remove verbosity
+            /*{
+            kDebug() << "dumpiiiiiing" << folder->url();
+            KDevelop::DUChainReadLocker lock(KDevelop::DUChain::lock());
+            KDevelop::DumpChain dump;
+            dump.dump( v.context(), false);
+            }*/
+
+            if(folder->text()=="/" && !v.projectName().isEmpty())
             {
+                folder->setText(v.projectName());
+            }
+
+            KUrl subroot=m_subprojectRoot[item->project()];
+            
+            foreach (const QString& subf, v.subdirectories())
+            {
+                if(subf.isEmpty()) //This would not be necessary if we didn't parse the wrong lines
+                    continue;
+                KUrl path(folder->url());
+                path.addPath(subf);
+                path.adjustPath(KUrl::AddTrailingSlash);
                 
-                QString outputName=t;
-                if(v.targetHasProperty(t, "OUTPUT_NAME"))
-                    outputName=v.targetProperty(t, "OUTPUT_NAME");
-             
-                KDevelop::ProjectTargetItem* targetItem;   
-                switch(v.targetType(t))
+                kDebug(9042) << "Found subdir " << path << "which should be into" << subroot;
+                if(subroot.isParentOf(path) || path.isParentOf(subroot))
                 {
-                    case CMakeProjectVisitor::Library:
-                        targetItem = new CMakeLibraryTargetItem( item->project(), t, folder, v.declarationsPerTarget()[t], outputName );
-                        break;
-                    case CMakeProjectVisitor::Executable:
-                        targetItem = new CMakeExecutableTargetItem( item->project(), t, folder, v.declarationsPerTarget()[t], outputName );
-                        break;
-                }
-                DescriptorAttatched* datt=dynamic_cast<DescriptorAttatched*>(targetItem);
-                datt->setDescriptor(v.targetDeclarationDescriptor(outputName));
-
-                foreach( const QString & sFile, dependencies )
-                {
-                    if(sFile.isEmpty())
-                        continue;
-
-                    KUrl sourceFile(sFile);
-                    if(sourceFile.isRelative()) {
-                        sourceFile = folder->url();
-                        sourceFile.adjustPath( KUrl::RemoveTrailingSlash );
-                        sourceFile.addPath( sFile );
-                    }
-
-                    if( entries.contains( sourceFile.fileName() ) )
+                    if(entries.contains(subf))
                     {
-                        entries.removeAll( sourceFile.fileName() );
+                        entries.removeAll(subf);
                     }
-
-                    new KDevelop::ProjectFileItem( item->project(), sourceFile, targetItem );
-                    item->project()->addToFileSet( KDevelop::IndexedString( sourceFile ) );
-                    kDebug(9042) << "..........Adding:" << sourceFile;
+                    
+                    CMakeFolderItem* a = new CMakeFolderItem( folder->project(), subf, folder );
+                    kDebug() << "folder: " << a << a->index();
+                    a->setUrl(path);
+                    a->setDefinitions(v.definitions());
+                    folderList.append( a );
+                    
+                    DescriptorAttatched* datt=static_cast<DescriptorAttatched*>(a);
+                    datt->setDescriptor(v.folderDeclarationDescriptor(subf));
                 }
-                m_targets.append(targetItem);
             }
-        }
+            
+    //         if(folderList.isEmpty() && path.isParentOf(item->url()))
+    //             kDebug() << "poor guess";
+            
+            QStringList directories;
+            directories += folder->url().toLocalFile(KUrl::RemoveTrailingSlash);
+            directories += currentBinDir;
 
+            foreach(const QString& s, v.includeDirectories())
+            {
+                QString dir(s);
+                if(!s.startsWith('/') && !s.startsWith("#["))
+                {
+                    KUrl path=folder->url();
+                    path.addPath(s);
+                    dir=path.toLocalFile();
+                }
+                
+                KUrl simp(dir); //We use this to simplify dir
+                simp.cleanPath();
+                dir=simp.toLocalFile();
+                
+                if(!directories.contains(dir))
+                    directories.append(dir);
+            }
+            folder->setIncludeDirectories(directories);
+            folder->setDefinitions(v.definitions());
+
+            foreach ( const QString& t, v.targets())
+            {
+                QStringList dependencies=v.targetDependencies(t);
+                if(!dependencies.isEmpty()) //Just to remove verbosity
+                {
+                    
+                    QString outputName=t;
+                    if(v.targetHasProperty(t, "OUTPUT_NAME"))
+                        outputName=v.targetProperty(t, "OUTPUT_NAME");
+                
+                    KDevelop::ProjectTargetItem* targetItem;   
+                    switch(v.targetType(t))
+                    {
+                        case CMakeProjectVisitor::Library:
+                            targetItem = new CMakeLibraryTargetItem( item->project(), t, folder, v.declarationsPerTarget()[t], outputName );
+                            break;
+                        case CMakeProjectVisitor::Executable:
+                            targetItem = new CMakeExecutableTargetItem( item->project(), t, folder, v.declarationsPerTarget()[t], outputName );
+                            break;
+                    }
+                    DescriptorAttatched* datt=dynamic_cast<DescriptorAttatched*>(targetItem);
+                    datt->setDescriptor(v.targetDeclarationDescriptor(outputName));
+
+                    foreach( const QString & sFile, dependencies )
+                    {
+                        if(sFile.isEmpty())
+                            continue;
+
+                        KUrl sourceFile(sFile);
+                        if(sourceFile.isRelative()) {
+                            sourceFile = folder->url();
+                            sourceFile.adjustPath( KUrl::RemoveTrailingSlash );
+                            sourceFile.addPath( sFile );
+                        }
+
+                        if( entries.contains( sourceFile.fileName() ) )
+                        {
+                            entries.removeAll( sourceFile.fileName() );
+                        }
+
+                        new KDevelop::ProjectFileItem( item->project(), sourceFile, targetItem );
+                        item->project()->addToFileSet( KDevelop::IndexedString( sourceFile ) );
+                        kDebug(9042) << "..........Adding:" << sourceFile;
+                    }
+                    m_targets.append(targetItem);
+                }
+            }
+
+        }
     }
 
     foreach( const QString& entry, entries )
@@ -603,20 +617,20 @@ QList<KDevelop::ProjectFolderItem*> CMakeProjectManager::parse( KDevelop::Projec
         if( item->hasFileOrFolder( entry ) )
             continue;
 
-        KUrl folderurl = item->url();
-        folderurl.addPath( entry );
+        KUrl fileurl = item->url();
+        fileurl.addPath( entry );
 
-        KUrl cache=folderurl;
+        KUrl cache=fileurl;
         cache.addPath("CMakeCache.txt");
-        if( QFileInfo( folderurl.toLocalFile() ).isDir())
+        if( QFileInfo( fileurl.toLocalFile() ).isDir())
         {
+            fileurl.adjustPath(KUrl::AddTrailingSlash);
             if(!QFile::exists(cache.toLocalFile()))
-                folderList.append(new KDevelop::ProjectFolderItem( item->project(), folderurl, item ));
+                folderList.append(new KDevelop::ProjectFolderItem( item->project(), fileurl, item ));
         }
         else
         {
-            new KDevelop::ProjectFileItem( item->project(), folderurl, item );
-            item->project()->addToFileSet( KDevelop::IndexedString( folderurl ) );
+            addFile(fileurl, item);
         }
     }
 
@@ -782,49 +796,46 @@ void CMakeProjectManager::reimport(CMakeFolderItem* fi)
 
 void CMakeProjectManager::dirtyFile(const QString & dirty)
 {
-    KUrl dir(dirty);
-    dir=dir.upUrl();
-    dir.adjustPath(KUrl::RemoveTrailingSlash);
-    KUrl dir2(dir);
-    dir2.adjustPath(KUrl::AddTrailingSlash);
-
-    CMakeFolderItem *it=0;
-    if(m_folderPerUrl.contains(dir))
-        it=m_folderPerUrl[dir];
-    else if(m_folderPerUrl.contains(dir2))
-        it=m_folderPerUrl[dir2];
-    else
-        kDebug(9032) << "Couldn't find the item for" << dir << dir2 << m_folderPerUrl;
-    kDebug(9042) << dir << " is dirty" << it << m_folderPerUrl;
-    Q_ASSERT(it);
-    KDevelop::IProject* proj=it->project();
-    KUrl projectBaseUrl=m_realRoot[proj];
-    projectBaseUrl.adjustPath(KUrl::RemoveTrailingSlash);
-
-    kDebug(9042) << "reload:" << dir << projectBaseUrl << (dir!=projectBaseUrl);
-    if(KUrl(dirty).fileName() == "CMakeLists.txt" && dir!=projectBaseUrl)
+    KUrl dirtyFile(dirty);
+    KUrl dir(dirtyFile.upUrl());
+    IProject* p=ICore::self()->projectController()->findProjectForUrl(dirtyFile);
+    
+    if(p && dirtyFile.fileName() == "CMakeLists.txt")
     {
-#if 0
-        KUrl relative=KUrl::relativeUrl(projectBaseUrl, dir);
-        initializeProject(proj, dir);
-        KUrl current=projectBaseUrl;
-        QStringList subs=relative.toLocalFile().split("/");
-        subs.append(QString());
-        for(; !subs.isEmpty(); current.cd(subs.takeFirst()))
+        QList<ProjectFileItem*> files=p->filesForUrl(dirtyFile);
+        kDebug(9042) << dir << " is dirty" << files;
+        
+        Q_ASSERT(!files.isEmpty());
+        CMakeFolderItem *it=static_cast<CMakeFolderItem*>(files.first()->parent());
+        
+        KDevelop::IProject* proj=it->project();
+        KUrl projectBaseUrl=m_realRoot[proj];
+        projectBaseUrl.adjustPath(KUrl::RemoveTrailingSlash);
+
+        kDebug(9042) << "reload:" << dir << projectBaseUrl << (dir!=projectBaseUrl);
+        if(dir!=projectBaseUrl)
         {
-            parseOnly(proj, current);
-        }
+#if 0
+            KUrl relative=KUrl::relativeUrl(projectBaseUrl, dir);
+            initializeProject(proj, dir);
+            KUrl current=projectBaseUrl;
+            QStringList subs=relative.toLocalFile().split("/");
+            subs.append(QString());
+            for(; !subs.isEmpty(); current.cd(subs.takeFirst()))
+            {
+                parseOnly(proj, current);
+            }
 #endif
-        QStandardItem *parent=it->parent();
-        parent->removeRow(it->row());
-        CMakeFolderItem* fi=new CMakeFolderItem( proj, dir.toLocalFile(), parent);
-        reimport(fi);
-        m_folderPerUrl[dir]=fi;
-    }
-    else if(it)
-    {
-        qDebug() << "reloading";
-        reload(proj->projectItem());
+            QStandardItem *parent=it->parent();
+            parent->removeRow(it->row());
+            CMakeFolderItem* fi=new CMakeFolderItem( proj, dir.toLocalFile(), parent);
+            reload(fi);
+        }
+        else
+        {
+    //         qDebug() << "reloading";
+            reload(proj->projectItem());
+        }
     }
     else
     {
@@ -937,6 +948,9 @@ KDevelop::ProjectFolderItem* CMakeProjectManager::addFolder( const KUrl& folder,
     KUrl lists=parent->url();
     lists.addPath("CMakeLists.txt");
     QString relative=KUrl::relativeUrl(parent->url(), folder);
+    
+    kDebug() << "Adding folder " << parent->url() << " to " << folder << " as " << relative;
+    
     Q_ASSERT(!relative.contains("/"));
 //     CMakeFileContent f = CMakeListsParser::readCMakeFile(file);
     
@@ -971,6 +985,11 @@ bool CMakeProjectManager::removeFolder( KDevelop::ProjectFolderItem* it)
 {
     KUrl lists=it->url().upUrl();
     lists.addPath("CMakeLists.txt");
+    if(it->type()!=KDevelop::ProjectBaseItem::BuildFolder)
+    {
+        it->parent()->removeRow(it->row());
+        return true;
+    }
     
     ApplyChangesWidget e(i18n("Remove a folder called '%1'.", it->text()), lists);
     CMakeFolderItem* cmit=static_cast<CMakeFolderItem*>(it);
@@ -988,4 +1007,149 @@ bool CMakeProjectManager::removeFolder( KDevelop::ProjectFolderItem* it)
     return 0;
 }
 
+bool followUses(KTextEditor::Document* doc, SimpleRange r, const QString& name, const KUrl& lists, bool add)
+{
+    bool ret=true;
+    QString txt=doc->text(r.textRange());
+    if(!add && txt.contains(name))
+    {
+        txt.replace(name, QString());
+        doc->replaceText(r.textRange(), txt);
+        ret=true;
+    }
+    else
+    {
+        KDevelop::DUChainReadLocker lock(KDevelop::DUChain::lock());
+        KDevelop::ReferencedTopDUContext topctx=DUChain::self()->chainForDocument(lists);
+        QList<Declaration*> decls;
+        for(int i=0; i<topctx->usesCount(); i++)
+        {
+            Use u = topctx->uses()[i];
+            
+            if(!r.contains(u.m_range))
+                continue; //We just want the uses in the range, not the whole file
+            
+            Declaration* d=u.usedDeclaration(topctx);
+            
+            if(d->context()->topContext()->url().toUrl()==lists)
+                decls += d;
+        }
+        
+        if(add && decls.isEmpty())
+        {
+            doc->insertText(r.textRange().start(), name);
+            ret=true;
+        }
+        else foreach(Declaration* d, decls)
+        {
+            r.start=d->range().end;
+            
+            for(int l=r.start.line; ;l++)
+            {
+                QString line=doc->line(l);
+                int c;
+                if((c=line.indexOf(')'))>=0) {
+                    r.end=SimpleCursor(l,c);
+                    break;
+                } else if(line.isEmpty()) {
+                    r=SimpleRange();
+                    break;
+                }
+            }
+            
+            if(!r.isEmpty())
+            {
+                ret = ret && followUses(doc, r, name, lists, add);
+            }
+        }
+    }
+    return ret;
+}
+
+bool CMakeProjectManager::removeFile( KDevelop::ProjectFileItem* it)
+{
+    bool ret=true;
+    if(!static_cast<ProjectBaseItem*>(it->parent())->target())
+    {
+        it->parent()->removeRow(it->row());
+    }
+    else
+    {
+        ProjectTargetItem* target=static_cast<ProjectTargetItem*>(it->parent());
+        ret = removeFileFromTarget(it, target);
+    }
+    return ret;
+}
+
+bool CMakeProjectManager::removeFileFromTarget( KDevelop::ProjectFileItem* it, KDevelop::ProjectTargetItem* target)
+{
+    if(it->parent()!=target)
+        return false; //It is not a cmake-managed file
+    
+    CMakeFolderItem* folder=static_cast<CMakeFolderItem*>(target->parent());
+    
+    DescriptorAttatched* desc=dynamic_cast<DescriptorAttatched*>(target);
+    SimpleRange r=desc->descriptor().range();
+    r.start=SimpleCursor(desc->descriptor().arguments.first().range().end);
+    
+    KUrl lists=folder->url();
+    lists.addPath("CMakeLists.txt");
+    
+    ApplyChangesWidget e(i18n("Remove a file called '%1'.", it->text()), lists);
+    
+    bool ret=followUses(e.document(), r, ' '+it->text(), lists, false);
+    if(ret)
+    {
+        if(e.exec())
+        {
+            bool saved=e.document()->documentSave();
+            if(!saved)
+                KMessageBox::error(0, i18n("KDevelop - CMake Support"),
+                                    i18n("Cannot save the change."));
+        }
+    }
+    else
+    {
+        KMessageBox::error(0, i18n("KDevelop - CMake Support"),
+                              i18n("Cannot remove the file."));
+    }
+    return ret;
+}
+
+//This is being called from ::parse() so we shouldn't make it block the ui
+KDevelop::ProjectFileItem* CMakeProjectManager::addFile( const KUrl& url, KDevelop::ProjectFolderItem* parent)
+{
+    ProjectFileItem* it = new KDevelop::ProjectFileItem( parent->project(), url, parent );
+    parent->project()->addToFileSet( KDevelop::IndexedString( url ) );
+    return it;
+}
+
+bool CMakeProjectManager::addFileToTarget( KDevelop::ProjectFileItem* it, KDevelop::ProjectTargetItem* target)
+{
+    if(it->parent()==target)
+        return true; //It already is in the target
+    
+    CMakeFolderItem* folder=static_cast<CMakeFolderItem*>(target->parent());
+    
+    DescriptorAttatched* desc=dynamic_cast<DescriptorAttatched*>(target);
+    SimpleRange r=desc->descriptor().range();
+    r.start=SimpleCursor(desc->descriptor().arguments.first().range().end);
+    
+    KUrl lists=folder->url();
+    lists.addPath("CMakeLists.txt");
+    
+    ApplyChangesWidget e(i18n("Add a file called '%1' to target '%2'.", it->text(), target->text()), lists);
+    
+    bool ret=followUses(e.document(), r, ' '+it->text(), lists, true);
+    
+    if(e.exec())
+    {
+        bool saved=e.document()->documentSave();
+        if(!saved)
+            KMessageBox::error(0, i18n("KDevelop - CMake Support"),
+                                  i18n("Cannot save the change."));
+    }
+    return ret;
+}
+
 #include "cmakemanager.moc"
