@@ -44,6 +44,8 @@
 #include <QFileInfo>
 #include <QScriptEngine>
 #include <QScriptValue>
+#include <language/editor/editorintegrator.h>
+#include <language/duchain/smartconverter.h>
 
 using namespace KDevelop;
 
@@ -88,7 +90,7 @@ QList< CMakeProjectVisitor::IntPair > CMakeProjectVisitor::parseArgument(const Q
                 gotDollar=true;
                 break;
             case '{':
-                if(gotDollar) 
+                if(gotDollar)
                 {
                     opened.push(i);
                 }
@@ -100,7 +102,7 @@ QList< CMakeProjectVisitor::IntPair > CMakeProjectVisitor::parseArgument(const Q
                 break;
         }
     }
-    
+
     for(int i=pos.count()-1; i>=0 && !opened.isEmpty(); i--)
     {
         if(pos[i].first==opened.top())
@@ -130,7 +132,7 @@ QStringList CMakeProjectVisitor::theValue(const QString& exp, const IntPair& the
     }
     else
         kDebug() << "error: I do not understand the key: " << type;
-    
+
 //     qDebug() << "solving: " << var << vars << exp;
     return vars;
 }
@@ -151,7 +153,7 @@ QStringList CMakeProjectVisitor::value(const QString& exp, const QList<IntPair>&
     {
         invars+=poss[desired+1];
     }
-    
+
     if(invars.count()>1)
     {
         QList<IntPair>::const_iterator itConstEnd=invars.constEnd();
@@ -162,14 +164,14 @@ QStringList CMakeProjectVisitor::value(const QString& exp, const QList<IntPair>&
             const IntPair& subvar=*it;
             int dollar=var.lastIndexOf('$', subvar.first);
             QString id=var.mid(dollar, subvar.second-dollar+1), value=theValue(var, subvar).join(QChar(';'));
-            
+
             int diff=value.size()-id.size();
             for(QList<IntPair>::iterator it=itBegin; it!=itEnd; ++it)
             {
                 if(it->first > subvar.first) it->first += diff;
                 if(it->second> subvar.second) it->second+= diff;
             }
-            
+
             var=replaceOne(var, id, value, dollar);
         }
     }
@@ -184,20 +186,20 @@ QStringList CMakeProjectVisitor::resolveVariable(const CMakeFunctionArgument &ex
 
     int i=0;
     IntPair last(-1,-1, 0);
-    
+
     for(QList<IntPair>::const_iterator it=var.constBegin(); it!=var.constEnd(); ++it)
     {
         while(it!=var.constEnd() && it->level>1)
             ++it;
-        
+
         const IntPair& p=*it;
 //         qDebug () << "reeeeeet" << ret << exp.value << p.print();
         int dollar=exp.value.lastIndexOf('$', p.first);
         QString pre=exp.value.mid(last.second+1, dollar-last.second-1);
-        
+
         QStringList vars = value(exp.value, var, i);
 //         qDebug() << "aaaaaaaaaA" << vars;
-        
+
         if(!vars.isEmpty())
         {
             pre+=vars.takeFirst();
@@ -208,7 +210,7 @@ QStringList CMakeProjectVisitor::resolveVariable(const CMakeFunctionArgument &ex
         i++;
     }
     ret.last().append(exp.value.mid(last.second+1, exp.value.count()-last.second));
-    
+
     if(exp.quoted)
     {
         ret=QStringList(ret.join(QChar(';')));
@@ -273,10 +275,10 @@ int CMakeProjectVisitor::visit( const SetTargetPropsAst * targetProps)
 int CMakeProjectVisitor::visit(const AddSubdirectoryAst *subd)
 {
     kDebug(9042) << "adding subdirectory" << subd->sourceDir();
-    
+
     VisitorState p=stackTop();
     m_folderDesc[subd->sourceDir()]=p.code->at(p.line);
-    
+
     m_subdirectories += subd->sourceDir();
     return 1;
 }
@@ -450,23 +452,7 @@ int CMakeProjectVisitor::visit(const IncludeAst *inc)
             ReferencedTopDUContext aux=m_topctx;
             if(m_topctx)
             {
-                DUChainWriteLocker lock(DUChain::lock());
-                m_topctx=DUChain::self()->chainForDocument(KUrl(include.first().filePath));
-                if(!m_topctx)
-                {
-                    m_topctx=new TopDUContext(IndexedString(KUrl(include.first().filePath).pathOrUrl()),
-                            SimpleRange(0,0, include.last().endColumn, include.last().endLine));
-                    DUChain::self()->addDocumentChain(m_topctx);
-
-                    Q_ASSERT(DUChain::self()->chainForDocument(KUrl(include.first().filePath)));
-                }
-                else
-                {
-                    m_topctx->deleteLocalDeclarations();
-                    m_topctx->deleteChildContextsRecursively();
-                    m_topctx->deleteUses();
-                }
-                aux->addImportedParentContext(m_topctx);
+                m_topctx=createContext(KUrl(include.first().filePath), aux, include.last().endLine-1, include.last().endColumn-1);
             }
             kDebug(9042) << "including:" << path;
             walk(include, 0);
@@ -504,7 +490,7 @@ int CMakeProjectVisitor::visit(const FindPackageAst *pack)
         return 1;
     QStringList modulePath = m_vars->value("CMAKE_MODULE_PATH") + m_modulePath;
     kDebug(9042) << "Find:" << pack->name() << "package." << m_modulePath << "No module: " << pack->noModule();
-    
+
     QStringList possibs;
     if(pack->noModule())
     {
@@ -514,9 +500,9 @@ int CMakeProjectVisitor::visit(const FindPackageAst *pack)
             instPath = m_vars->value(var)[0];
         else if(m_cache->contains(var))
             instPath = m_cache->value(var);
-        
+
         kDebug(9042) << "config mode" << m_vars->value(var)[0] << m_cache->value(var) << instPath;
-        
+
 #if defined(Q_OS_WIN)
         modulePath.prepend(instPath);
         modulePath.prepend(instPath+"/cmake");
@@ -529,7 +515,7 @@ int CMakeProjectVisitor::visit(const FindPackageAst *pack)
             modulePath.prepend(instPath+"/lib/"+name.toLower()+post);
         }
 #endif
-        
+
         possibs+=QString("%1Config.cmake").arg(pack->name());
         possibs+=QString("%1-config.cmake").arg(pack->name().toLower());
     }
@@ -541,7 +527,7 @@ int CMakeProjectVisitor::visit(const FindPackageAst *pack)
         possib.prepend("Find");
         possibs += possib;
     }
-    
+
     QString path;
     foreach(const QString& possib, possibs)
     {
@@ -549,7 +535,7 @@ int CMakeProjectVisitor::visit(const FindPackageAst *pack)
         if(!path.isEmpty())
             break;
     }
-    
+
     if(!path.isEmpty())
     {
         m_vars->insertMulti("CMAKE_CURRENT_LIST_FILE", QStringList(path));
@@ -563,25 +549,7 @@ int CMakeProjectVisitor::visit(const FindPackageAst *pack)
             path=KUrl(path).pathOrUrl();
             kDebug(9042) << "================== Found" << path << "===============";
             ReferencedTopDUContext aux=m_topctx;
-            {
-                DUChainWriteLocker lock(DUChain::lock());
-                m_topctx=DUChain::self()->chainForDocument(KUrl(path));
-                if(!m_topctx)
-                {
-                    m_topctx=new TopDUContext(IndexedString(path),
-                            SimpleRange(0,0, package.last().endColumn, package.last().endLine));
-                    DUChain::self()->addDocumentChain(m_topctx);
-
-                    Q_ASSERT(DUChain::self()->chainForDocument(KUrl(path)));
-                    aux->addImportedParentContext(m_topctx);
-                }
-                else
-                {
-                    m_topctx->deleteLocalDeclarations();
-                    m_topctx->deleteChildContextsRecursively();
-                    m_topctx->deleteUses();
-                }
-            }
+            m_topctx=createContext(KUrl(path), aux, package.last().endColumn, package.last().endLine);
             walk(package, 0);
             m_topctx=aux;
         }
@@ -589,7 +557,7 @@ int CMakeProjectVisitor::visit(const FindPackageAst *pack)
         {
             kDebug(9032) << "error: find_package. Parsing error." << path;
         }
-        
+
         if(pack->noModule())
         {
             m_vars->insert(QString("%1_CONFIG"), QStringList(path));
@@ -602,10 +570,38 @@ int CMakeProjectVisitor::visit(const FindPackageAst *pack)
         kDebug(9032) << "error: Could not find" << pack->name() << "into" << modulePath;
     }
     kDebug(9042) << "Exit. Found:" << pack->name() << m_vars->value(pack->name()+"_FOUND");
-    
+
     return 1;
 }
 
+KDevelop::ReferencedTopDUContext CMakeProjectVisitor::createContext(const KUrl& path, ReferencedTopDUContext aux, int endl ,int endc)
+{
+    DUChainWriteLocker lock(DUChain::lock());
+    KDevelop::ReferencedTopDUContext topctx=DUChain::self()->chainForDocument(path);
+    if(!topctx)
+    {
+        topctx=new TopDUContext(IndexedString(path),
+                SimpleRange(0,0, endl, endc));
+        DUChain::self()->addDocumentChain(topctx);
+
+        Q_ASSERT(DUChain::self()->chainForDocument(path));
+        aux->addImportedParentContext(topctx);
+    }
+    else
+    {
+        topctx->deleteLocalDeclarations();
+        topctx->deleteChildContextsRecursively();
+        topctx->deleteUses();
+        
+        EditorIntegrator editor;
+        editor.setCurrentUrl(IndexedString(topctx->url().toUrl()));
+        
+        SmartConverter converter(&editor);
+        converter.deconvertDUChain(topctx);
+    }
+    return topctx;
+}
+
 bool CMakeProjectVisitor::haveToFind(const QString &varName)
 {
     if(m_vars->contains(varName+"_FOUND"))
@@ -680,7 +676,7 @@ int CMakeProjectVisitor::visit(const FindPathAst *fpath)
     }
 
     bool error=false;
-    QStringList locationOptions = fpath->path();
+    QStringList locationOptions = fpath->path()+fpath->hints();
     QStringList path, files=fpath->filenames();
 
     if(!fpath->noDefaultPath()) {
@@ -690,7 +686,7 @@ int CMakeProjectVisitor::visit(const FindPathAst *fpath)
     kDebug(9042) << "Find:" << /*locationOptions << "@" <<*/ fpath->variableName() << /*"=" << files <<*/ " path.";
     foreach(const QString& p, files)
     {
-        QString p1=findFile(p, fpath->hints()+locationOptions, fpath->pathSuffixes(), true);
+        QString p1=findFile(p, locationOptions, fpath->pathSuffixes(), true);
         if(p1.isEmpty())
         {
             kDebug(9042) << p << "not found";
@@ -727,13 +723,14 @@ int CMakeProjectVisitor::visit(const FindLibraryAst *flib)
     }
 
     bool error=false;
-    QStringList locationOptions = flib->path();
+    QStringList locationOptions = flib->path()+flib->hints();
     QStringList files=flib->filenames();
     QString path;
 
     if(!flib->noDefaultPath())
     {
         locationOptions += m_defaultPaths;
+        locationOptions += m_vars->value("CMAKE_INSTALL_PREFIX").join(QString())+"/lib";
     }
 
     foreach(const QString& p, files)
@@ -742,7 +739,7 @@ int CMakeProjectVisitor::visit(const FindLibraryAst *flib)
         {
             foreach(const QString& suffix, m_vars->value("CMAKE_FIND_LIBRARY_SUFFIXES"))
             {
-                QString p1=findFile(prefix+p+suffix, flib->hints()+locationOptions, flib->pathSuffixes());
+                QString p1=findFile(prefix+p+suffix, locationOptions, flib->pathSuffixes());
                 if(p1.isEmpty())
                 {
                     kDebug(9042) << p << "not found";
@@ -784,13 +781,13 @@ int CMakeProjectVisitor::visit(const FindFileAst *ffile)
     }
 
     bool error=false;
-    QStringList locationOptions = ffile->path();
+    QStringList locationOptions = ffile->path()+ffile->hints();
     QStringList path, files=ffile->filenames();
 
     kDebug(9042) << "Find File:" << ffile->filenames();
     foreach(const QString& p, files)
     {
-        QString p1=findFile(p, ffile->hints()+locationOptions, ffile->pathSuffixes());
+        QString p1=findFile(p, locationOptions, ffile->pathSuffixes());
         if(p1.isEmpty())
         {
             kDebug(9042) << p << "not found";
@@ -855,7 +852,7 @@ void CMakeProjectVisitor::macroDeclaration(const CMakeFunctionDesc& def, const C
     {
         Declaration *d = new Declaration(sr, m_topctx);
         d->setIdentifier( Identifier(id) );
-        
+
         FunctionType* func=new FunctionType();
         foreach(const QString& arg, args)
         {
@@ -916,7 +913,7 @@ int CMakeProjectVisitor::visit(const FunctionAst *func)
         ++lines;
     }
     ++lines; //We do not want to return to endmacro
-    
+
     if(it!=itEnd)
     {
         m_macros->insert(func->name(), m);
@@ -951,7 +948,7 @@ int CMakeProjectVisitor::visit(const MacroCallAst *call)
                     m_topctx->createUse(idx, call->content()[call->line()].nameRange(), 0);
                 }
             }
-            
+
             //Giving value to parameters
             QStringList::const_iterator mit = code.knownArgs.constBegin();
             QStringList::const_iterator cit = call->arguments().constBegin();
@@ -1015,7 +1012,7 @@ void usesForArguments(const QStringList& names, const QList<int>& args, const Re
     //TODO: Should not return here
     if(args.size()!=names.size())
         return;
-        
+
     //We define the uses for the used variable without ${}
     foreach(int use, args)
     {
@@ -1023,7 +1020,7 @@ void usesForArguments(const QStringList& names, const QList<int>& args, const Re
         QString var=names[use];
         QList<Declaration*> decls=topctx->findDeclarations(Identifier(var));
 
-        if(!decls.isEmpty())
+        if(!decls.isEmpty() && func.arguments.count() > use)
         {
             CMakeFunctionArgument arg=func.arguments[use];
             int idx=topctx->indexForUsedDeclaration(decls.first());
@@ -1035,7 +1032,7 @@ void usesForArguments(const QStringList& names, const QList<int>& args, const Re
 int CMakeProjectVisitor::visit(const IfAst *ifast)  //Highly crappy code
 {
     int lines=ifast->line();
-    if( ifast->condition().isEmpty() ) 
+    if( ifast->condition().isEmpty() )
     {
         const CMakeFunctionDesc d = ifast->content().at( ifast->line() );
         kWarning() << "Parser couldn't parse condition of an IF in file:" << ifast->condition() << d.filePath << d.line;
@@ -1058,39 +1055,41 @@ int CMakeProjectVisitor::visit(const IfAst *ifast)  //Highly crappy code
         }
         else if(funcName=="endif")
         {
-            inside--;
-            if(inside<=0 && !it->arguments.isEmpty()) {
+            inside--; 
+            if(inside<=0) {
 //                 Q_ASSERT(!ini.isEmpty());
-                usesForArguments(ifast->condition(), ini, m_topctx, *it);     
+                if(!it->arguments.isEmpty())
+                    usesForArguments(ifast->condition(), ini, m_topctx, *it);
                 break;
             }
 //                 kDebug(9042) << "found an endif at:" << lines << "but" << inside;
         }
-        
+
         if(inside==1)
         {
             bool result;
-            
+
             if(funcName=="if" || funcName=="elseif")
             {
                 CMakeCondition cond(this);
                 IfAst myIf;
                 QStringList condition;
-                
-                if(funcName!="if")
+
+                if(funcName=="if")
+                {
+                    condition=ifast->condition();
+                }
+                else
                 {
                     if(!myIf.parseFunctionInfo(resolveVariables(*it)))
                         kDebug(9042) << "uncorrect condition correct" << it->writeBack();
                     condition=myIf.condition();
                 }
-                else
-                {
-                    condition=ifast->condition();
-                    ini=cond.variableArguments();
-                }
                 result=cond.condition(condition);
-                    
-                usesForArguments(myIf.condition(), cond.variableArguments(), m_topctx, *it);
+                if(funcName=="if")
+                    ini=cond.variableArguments();
+
+                usesForArguments(condition, cond.variableArguments(), m_topctx, *it);
                 kDebug(9042) << ">> " << funcName << condition << result;
             }
             else if(funcName=="else")
@@ -1099,22 +1098,22 @@ int CMakeProjectVisitor::visit(const IfAst *ifast)  //Highly crappy code
                 result=true;
                 usesForArguments(ifast->condition(), ini, m_topctx, *it);
             }
-            
+
             if(!visited && result)
             {
                 kDebug(9042) << "About to visit " << funcName << "?" << result;
-                
+
                 int oldpos=lines;
                 lines = walk(ifast->content(), lines+1)-1;
-                
+
                 it+=lines-oldpos;
-                
+
                 visited=true;
 //                 kDebug(9042) << "Visited. now in" << it->name;
             }
         }
     }
-    
+
     if(it==itEnd)
     {
         kDebug() << "error. found an unfinished endif";
@@ -1534,7 +1533,7 @@ int CMakeProjectVisitor::visit(const ForeachAst *fea)
 
 int CMakeProjectVisitor::visit(const StringAst *sast)
 {
-    kDebug(9042) << "String to" /*<< sast->input()*/ << sast->input().isEmpty();
+    kDebug(9042) << "String to" /*<< sast->input()*/ << sast->input();
     switch(sast->type())
     {
         case StringAst::REGEX:
@@ -1565,25 +1564,20 @@ int CMakeProjectVisitor::visit(const StringAst *sast)
                 }
                 case StringAst::REGEX_REPLACE:
                 {
-                    foreach(const QString& _in, sast->input())
+                    foreach(const QString& in, sast->input())
                     {
-                        QString in(_in);
-                        
-                        int idx = rx.indexIn(in);
+                        rx.indexIn(in);
                         QStringList info = rx.capturedTexts();
-                        if(idx<0)
+                        int idx=0, count=0, last=0;
+                        
+                        QString tmp = sast->replace();
+                        for(int i = 1; i < info.count(); i++)
                         {
-                            res.append(in);
-                        }
-                        else
-                        {
-                            QString tmp = sast->replace();
-                            for(int i = 1; i < info.count(); i++)
-                            {
-                                tmp.replace(QString("\\%1").arg(i), info.at(i));
-                            }
-                            res.append(in.left(idx)+tmp+in.right(in.length()-idx-rx.matchedLength()));
+                            tmp.replace(QString("\\%1").arg(i), info.at(i));
                         }
+                        
+                        QString toret=in;
+                        res.append(toret.replace(rx, tmp));
                     }
                 }
                     break;
@@ -1591,7 +1585,7 @@ int CMakeProjectVisitor::visit(const StringAst *sast)
                     kDebug(9032) << "ERROR String: Not a regex. " << sast->cmdType();
                     break;
             }
-            kDebug() << "regex" << sast->regex() << res;
+            kDebug() << "regex" << sast->outputVariable() << "=" << sast->regex() << res;
             m_vars->insert(sast->outputVariable(), res);
         }
             break;
@@ -1637,7 +1631,7 @@ int CMakeProjectVisitor::visit(const StringAst *sast)
                 bool ok;
                 res += QChar(ascii.toInt(&ok, 10));
             }
-            
+
             m_vars->insert(sast->outputVariable(), QStringList(res));
         }   break;
         case StringAst::CONFIGURE:
@@ -1795,7 +1789,7 @@ int CMakeProjectVisitor::visit( const WhileAst * whileast)
     CMakeCondition cond(this);
     bool result=cond.condition(whileast->condition());
     usesForArguments(whileast->condition(), cond.variableArguments(), m_topctx, whileast->content()[whileast->line()]);
-    
+
     kDebug(9042) << "Visiting While" << whileast->condition() << "?" << result;
     if(result)
     {
@@ -1813,7 +1807,7 @@ int CMakeProjectVisitor::visit( const WhileAst * whileast)
         else if(funcName=="endwhile")
             inside--;
     }
-    
+
     if(it!=itEnd) {
         usesForArguments(whileast->condition(), cond.variableArguments(), m_topctx, *(it-1));
     }
@@ -1854,20 +1848,8 @@ int CMakeProjectVisitor::walk(const CMakeFileContent & fc, int line)
 {
     if(!m_topctx)
     {
-        DUChainWriteLocker lock(DUChain::lock());
         KUrl url(fc[0].filePath);
-        IndexedString pathOrUrl(url.pathOrUrl());
-        m_topctx=DUChain::self()->chainForDocument(pathOrUrl);
-        if(!m_topctx)
-        {
-            m_topctx=new TopDUContext(IndexedString(pathOrUrl.str()),
-                    SimpleRange(0,0, fc.last().endLine-1, fc.last().endColumn-1));
-
-            DUChain::self()->addDocumentChain(m_topctx);
-            Q_ASSERT(DUChain::self()->chainForDocument(pathOrUrl));
-        }
-        
-        m_topctx->addImportedParentContext(m_parentCtx);
+        m_topctx=createContext(url, m_parentCtx, fc.last().endLine-1, fc.last().endColumn-1);
     }
     VisitorState p;
     p.code = &fc;
@@ -1961,7 +1943,7 @@ void CMakeProjectVisitor::createUses(const CMakeFunctionDesc& desc)
     {
         if(!arg.isCorrect() || !arg.value.contains('$'))
             continue;
-        
+
         QList< IntPair > var = parseArgument(arg.value);
         for(QList<IntPair>::const_iterator it=var.constBegin(); it!=var.constEnd(); ++it)
         {
